/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl;

import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.wordsource.WordSourceId;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.UnknownNamespaceException;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;


/**
 * This class contains utility methods used by the implementation of this component. This class needs to be public as
 * multiple packages of the component use the functionality provided by this class.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class Utils {
    /**
     * This constant is used as initial size for newly created {@link java.io.ByteArrayOutputStream}s during
     * serialization in {@link #serializeObject(Object)}.
     */
    private static final int ONE_KILOBYTE = 1024;
    /**
     * These are the hex digits used in hex encoding a byte[] in {@link #createIdString(WordSourceId)}.
     */
    private static final char[] HEX_DIGITS = {
        '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
    /**
     * This constant is used as a mask to mask any high bytes of an int, so that the int is only made from the lowest
     * byte.
     */
    private static final int ONE_BYTE_MASK = 0xff;
    /**
     * This mask is used when splitting a byte into its two nibbles for encoding them into hex digits.
     */
    private static final int NIBBLE_MASK = 16;

    /**
     * As this class is a utility class, a private constructor is added to avoid class instantiation.
     */
    private Utils() {
    }

    /**
     * This method looks up a configuration property from the {@link com.topcoder.util.config.ConfigManager} and parses
     * the value retrieved into an <tt>int</tt> value that is at least the specified minValue.
     *
     * @param namespace A non-<tt>null</tt>, non-empty (trim'd) string representing the {@link
     *                  com.topcoder.util.config.ConfigManager} namespace in which to lookup the configuration property
     * @param key       A non-<tt>null</tt>, non-empty (trim'd) string representing the name of the property to be
     *                  looked up
     * @param minValue  the minimum value of the <tt>int</tt> value parsed from the value looked up, a {@link
     *                  PersistenceConfigurationException} is thrown in case the value parsed is less than this value
     *
     * @return the <tt>int</tt> value looked up from the {@link com.topcoder.util.config.ConfigManager}, <tt>&gt;=</tt>
     *         the given minValue
     *
     * @throws IllegalArgumentException if namespace or key are empty (trim'd) strings
     * @throws PersistenceConfigurationException
     *                                  in case the property lookup fails, returns <tt>null</tt>, is not parseable into
     *                                  an <tt>int</tt> or is <tt>&lt;minValue</tt>
     */
    public static int lookupIntFromConfigManager(final String namespace, final String key, final int minValue)
        throws PersistenceConfigurationException {
        final String value = lookupValidStringFromConfigManager(namespace, key);
        final int ret;
        try {
            ret = Integer.parseInt(value);
        } catch (NumberFormatException e) {
            throw new PersistenceConfigurationException(
                "The configuration value [" + value + "] for the property [" + key
                    + "] in namespace [" + namespace + "] was not parseable into a valid number.", e);
        }
        if (ret < minValue) {
            throw new PersistenceConfigurationException("The configuration value [" + ret + "] for the property [" + key
                + "] in namespace [" + namespace + "] was less than the minimum allowed value [" + minValue + "].");
        }
        return ret;
    }

    /**
     * This method looks up a configuration property of type {@link String} from the {@link ConfigManager}. The value
     * looked up is checked to be non-<tt>null</tt> an non-empty (trim'd).
     *
     * @param namespace A non-<tt>null</tt>, non-empty (trim'd) string representing the {@link ConfigManager} namespace
     *                  in which to lookup the configuration property
     * @param key       A non-<tt>null</tt>, non-empty (trim'd) string representing the name of the property to be
     *                  looked up
     *
     * @return the String value looked up from the {@link ConfigManager}
     *
     * @throws IllegalArgumentException if namespace or key are <tt>null</tt> or empty (trim'd) strings
     * @throws PersistenceConfigurationException
     *                                  in case the given namespace is unknown, the property lookup fails or returns
     *                                  <tt>null</tt>, or an empty (trim'd) string
     */
    public static String lookupValidStringFromConfigManager(final String namespace, final String key) throws
        PersistenceConfigurationException {
        // arg checking done in internalLookupStringFromConfigManager
        final String value;
        try {
            value = internalLookupStringFromConfigManager(namespace, key);
        } catch (UnknownNamespaceException e) {
            throw new PersistenceConfigurationException(
                "The given namespace [" + namespace + "] did not exist in ConfigManager.",
                e);
        }
        if (value == null) {
            throw new PersistenceConfigurationException(
                "The configuration value for the property [" + key + "] in namespace [" + namespace + "] was null.");
        }
        if (value.trim().length() == 0) {
            throw new PersistenceConfigurationException("The configuration value for the property [" + key
                + "] in namespace [" + namespace + "] was an empty String.");
        }
        return value;
    }

    /**
     * This method looks up an optional configuration property of type {@link String} from the {@link ConfigManager}.
     *
     * @param namespace A non-<tt>null</tt>, non-empty (trim'd) string representing the {@link ConfigManager} namespace
     *                  in which to lookup the configuration property
     * @param key       A non-<tt>null</tt>, non-empty (trim'd) string representing the name of the property to be
     *                  looked up
     *
     * @return the String value looked up from the {@link ConfigManager} or <tt>null</tt> if the value cannot be found,
     *         or was <tt>null</tt> or an empty String
     *
     * @throws IllegalArgumentException if namespace or key are <tt>null</tt> or empty (trim'd) strings
     */
    public static String lookupOptionalStringFromConfigManager(final String namespace, final String key) {
        // arg checking done in internalLookupStringFromConfigManager
        try {
            final String value = internalLookupStringFromConfigManager(namespace, key);
            if (value != null && value.trim().length() == 0) {
                return null;
            }
            return value;
        } catch (UnknownNamespaceException e) {
            return null;
        }
    }

    /**
     * This method serializes the given object.
     *
     * @param object the object to be serialized
     *
     * @return the serialized form of the given object
     *
     * @throws IndexPersistenceException in case the serialization fails
     * @throws IllegalArgumentException  in case the given object is <tt>null</tt>
     */
    public static byte[] serializeObject(final Object object) throws IndexPersistenceException {
        if (object == null) {
            throw new IllegalArgumentException("The parameter named [object] was null.");
        }

        final ByteArrayOutputStream baos = new ByteArrayOutputStream(ONE_KILOBYTE);
        try {
            final ObjectOutputStream objectOutputStream = new ObjectOutputStream(baos);
            objectOutputStream.writeObject(object);
            objectOutputStream.close();
            return baos.toByteArray();
        } catch (IOException e) {
            throw new IndexPersistenceException("Unable to serialize the given Object [" + object + "] of type ["
                + object.getClass().getName() + "].", e);
        }
        // no closing of streams needed as the underlying
        // streams are ByteArrayOutputStreams,
        // which are explicitly declared as not necessary to be closed.
    }

    /**
     * This method deserializes an {@link Object} from the given bytes.
     *
     * @param bytes the bytes to be deserialized into an {@link Object}
     *
     * @return the deserialized {@link Object}
     *
     * @throws IndexPersistenceException in case the deserialization fails
     * @throws IllegalArgumentException  in case the given bytes is <tt>null</tt>
     */
    public static Object deSerializeObject(final byte[] bytes) throws IndexPersistenceException {
        if (bytes == null) {
            throw new IllegalArgumentException("The parameter named [bytes] was null.");
        }
        final InputStream in = new ByteArrayInputStream(bytes);

        try {
            final ObjectInputStream objectInputStream = new ObjectInputStream(in);
            return objectInputStream.readObject();
            // no closing of streams needed as the underlying
            // streams are ByteArrayOutputStreams and ByteArrayInputStreams,
            // which are explicitly declared as not necessary to be closed.
        } catch (Exception e) {
            // catch of Exception is normally discouraged by the TC coding style, but
            // as a lot of Exceptions can arise from a deserialization
            // all Exceptions are caught in
            // this single catch clause, as no specialized handling
            // of different exception types is possible.
            throw new IndexPersistenceException("Error while deserializing Object from byte[].", e);
        }
    }

    /**
     * This method does create a unique String identifier from the given WordSourceId by calculating the MD5 hash sum of
     * the binary representation of the given WordSourceId and returning the Hex-encoding String of that hash code.
     *
     * @param sourceId the WordSourceId to create the id string for
     *
     * @return the id string of the given WordSourceId
     *
     * @throws IndexPersistenceException in case the creation of the id string fails
     * @throws IllegalArgumentException  in case the given sourceId is <tt>null</tt>
     */
    public static String createIdString(final WordSourceId sourceId) throws IndexPersistenceException {
        if (sourceId == null) {
            throw new IllegalArgumentException("The parameter named [sourceId] was null.");
        }

        // serialize the complete state of the wordSourceId to a byte[]
        final ByteArrayOutputStream baos = new ByteArrayOutputStream(ONE_KILOBYTE);
        final byte[] bytes;
        try {
            final ObjectOutputStream objectOutputStream = new ObjectOutputStream(baos);
            objectOutputStream.writeObject(sourceId.getSourceIdentity());
            objectOutputStream.writeObject(sourceId.getSourceClassName());
            objectOutputStream.writeObject(sourceId.getSourceLocale());
            objectOutputStream.writeObject(sourceId.getDelimiters());
            objectOutputStream.close();
            bytes = baos.toByteArray();
        } catch (IOException e) {
            throw new IndexPersistenceException(
                "Unable to serialize the WordSourceId [" + sourceId + "] for digesting.", e);
        }

        // Create an MD5 message digest
        try {
            // Create an MD5 message digest
            final MessageDigest msgDigest = MessageDigest.getInstance("MD5");
            msgDigest.update(bytes);
            final byte[] digest = msgDigest.digest();

            //now do a HEX encoding and return the String
            final char[] hex = new char[digest.length * 2]; //we need 2 characters per byte
            for (int i = 0; i < digest.length; i++) {
                final int b = digest[i] & ONE_BYTE_MASK;
                hex[2 * i] = HEX_DIGITS[b / NIBBLE_MASK];
                hex[2 * i + 1] = HEX_DIGITS[b % NIBBLE_MASK];
            }
            return new String(hex);
        } catch (NoSuchAlgorithmException e) {
            throw new IndexPersistenceException("Unable to digest the serialized WordSourceId [" + sourceId + "].", e);
        }
    }

    /**
     * This method looks up a configuration property of type {@link String} from the {@link ConfigManager}.
     *
     * @param namespace A non-<tt>null</tt>, non-empty (trim'd) string representing the {@link ConfigManager} namespace
     *                  in which to lookup the configuration property
     * @param key       A non-<tt>null</tt>, non-empty (trim'd) string representing the name of the property to be
     *                  looked up
     *
     * @return the String value looked up from the {@link ConfigManager} or <tt>null</tt> if the value cannot be found
     *
     * @throws IllegalArgumentException  if namespace or key are <tt>null</tt> or empty (trim'd) strings
     * @throws UnknownNamespaceException in case the given namespace does not exist in ConfigManager
     */
    private static String internalLookupStringFromConfigManager(final String namespace, final String key) throws
        UnknownNamespaceException {
        if (namespace == null) {
            throw new IllegalArgumentException("The parameter named [namespace] was null.");
        }
        if (key == null) {
            throw new IllegalArgumentException("The parameter named [key] was null.");
        }
        if (namespace.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [namespace] was an empty String.");
        }
        if (key.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [key] was an empty String.");
        }

        return ConfigManager.getInstance().getString(namespace, key);
    }

}
