/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.Utils;
import com.topcoder.document.index.wordsource.WordSourceId;

import java.sql.Connection;
import java.text.CollationKey;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;


/**
 * This class contains the database operations that are common to both database persistence implementations. These
 * methods have been factored out to reduce code redundancy. As these operation are quite specific to the persistence
 * layer they have been factored into a different utility class instead of adding the operations to {@link
 * DatabaseUtils} which does contain the more general and generic JDBC handling code.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
class CommonDatabaseOperations {
    /**
     * As this class is a utility class, a private constructor is added to avoid class instantiation.
     */
    private CommonDatabaseOperations() {
        //an empty private default constructor
    }

    /**
     * This method does query the database for a WordSourceId matching the given documentId String as described in CS
     * 1.4.1.1.
     *
     * @param persistence the AbstractDBIndexPersistence instance that can be used to perform rollback operations on the
     *                    given connection in case an error is encountered
     * @param connection  the connection to perform the necessary queries on
     * @param documentId  the document record id from which to create the WordSourceId
     *
     * @return the WordSourceId created for the given document id or <tt>null</tt> if no matching record existed in the
     *         database
     *
     * @throws IndexPersistenceException in case an error is encountered during database operations
     * @throws IllegalArgumentException  in case any arg is null or documentId is an empty (trim'd) String
     */
    static WordSourceId getWordSourceId(final AbstractDBIndexPersistence persistence,
                                        final Connection connection, final String documentId)
        throws IndexPersistenceException {
        if (documentId == null) {
            throw new IllegalArgumentException("The parameter named [documentId] was null.");
        }
        if (documentId.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [documentId] was an empty String.");
        }

        final Object[] queryArgsDocumentId = new Object[]{documentId};

        final List result1 = DatabaseUtils.doQuery(connection, persistence,
            "SELECT LOCALE,SOURCE_IDENTITY,CLASS_NAME FROM DOCUMENT WHERE DOCUMENT_ID=?",
            queryArgsDocumentId,
            new DatabaseUtils.DataType[]{
                DatabaseUtils.STRING_TYPE,
                DatabaseUtils.BYTE_ARRAY_TYPE,
                DatabaseUtils.STRING_TYPE}
        );

        if (result1.isEmpty()) {
            // no record matched the id, so we return null here
            return null;
        }

        final Object[] documentData = (Object[]) result1.get(0);
        final Object sourceIdentity = Utils.deSerializeObject((byte[]) documentData[1]);

        // build the locale from the locale string
        final String[] localeElements = ((String) documentData[0]).split("_");
        final Locale locale;
        switch (localeElements.length) {
        case 1:
            locale = new Locale(localeElements[0]);
            break;
        case 2:
            locale = new Locale(localeElements[0], localeElements[1]);
            break;
        case 3:
            locale = new Locale(localeElements[0], localeElements[1], localeElements[2]);
            break;
        default:
            persistence.rollbackTransaction(connection);
            throw new IndexPersistenceException("Unexpected locale String [" + documentData[0]
                + "] encountered, was not parseable into a valid locale.");
        }

        // fetch all delimiters
        final List result2 = DatabaseUtils.doSingleColumnQuery(connection, persistence,
            "SELECT DELIMITER_ENTRY FROM DELIMITER WHERE DOCUMENT_ID=? ORDER BY DELIMITER_SORT_ORDER ASC",
            queryArgsDocumentId,
            DatabaseUtils.STRING_TYPE
        );
        final String[] delimiters = (String[]) result2.toArray(new String[result2.size()]);

        //build WordSourceId
        return new WordSourceId(sourceIdentity, (String) documentData[2], delimiters, locale);
    }

    /**
     * This method does store the given document collection index in the persistence.
     * <p/>
     * CS section 1.4.3.1 describes the algorithm of this method.
     *
     * @param collectionIndex CollectionIndex to store
     * @param persistence     the AbstractDBIndexPersistence instance that can be used to obtain a database connection,
     *                        perform rollback operations on the connection in case an error is encountered or commit
     *                        the connection in case the operation has succeeded
     *
     * @throws IllegalArgumentException  if any arg is <tt>null</tt>
     * @throws IndexPersistenceException if fails to store document collection index or given index is already persisted
     *                                   or contains an unpersisted document index
     */
    static void addCollectionIndex(final AbstractDBIndexPersistence persistence, final CollectionIndex collectionIndex)
        throws
        IndexPersistenceException {
        if (collectionIndex == null) {
            throw new IllegalArgumentException("The parameter named [collectionIndex] was null.");
        }
        if (persistence == null) {
            throw new IllegalArgumentException("The parameter named [persistence] was null.");
        }
        final String collectionId = collectionIndex.getId();
        final Object[] collectionIdQueryArgs = new Object[]{collectionId};
        final Connection connection = persistence.getConnection();
        persistence.beginTransaction(connection);

        final Integer count = (Integer) DatabaseUtils.doSingleValueQuery(connection, persistence,
            "SELECT COUNT(*) FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", collectionIdQueryArgs,
            DatabaseUtils.INTEGER_TYPE);
        if (count.intValue() > 0) {
            DatabaseUtils.closeSilently(connection);
            throw new IndexPersistenceException(
                "The given collection with the id [" + collectionId + "] does already exist in the persistence.");
        }
        final Set allDocumentIds = collectionIndex.getAllDocumentIds();
        for (Iterator iterator = allDocumentIds.iterator(); iterator.hasNext();) {
            final WordSourceId sourceId = (WordSourceId) iterator.next();
            final String documentId = Utils.createIdString(sourceId);
            final Integer docCount = (Integer) DatabaseUtils.doSingleValueQuery(connection, persistence,
                "SELECT COUNT(*) from DOCUMENT WHERE DOCUMENT_ID=?",
                new Object[]{documentId}, DatabaseUtils.INTEGER_TYPE);
            if (docCount.intValue() == 0) {
                throw new IndexPersistenceException(
                    "The given document index collection contains unpersisted documents.");
            }
        }

        DatabaseUtils.doDMLQuery(connection, persistence,
            "INSERT INTO DOCUMENT_COLLECTION (COLLECTION_ID) VALUES (?)",
            collectionIdQueryArgs);

        for (Iterator iterator = allDocumentIds.iterator(); iterator.hasNext();) {
            final WordSourceId sourceId = (WordSourceId) iterator.next();
            final String documentId = Utils.createIdString(sourceId);
            DatabaseUtils.doDMLQuery(connection, persistence,
                "INSERT INTO DOCUMENT_COLLECTION_DOCUMENT_XREF (COLLECTION_ID,DOCUMENT_ID) VALUES (?,?)",
                new Object[]{collectionId, documentId});
            // Use count is maintained externally
        }

        persistence.commitTransaction(connection);
    }

    /**
     * This method retrieves a document collection index with specified identifier. It does return <tt>null</tt> when
     * collection with specified identifier is not found.
     * <p/>
     * CS section 1.4.3.2 describes the algorithm of this method.
     *
     * @param persistence  the AbstractDBIndexPersistence instance that can be used to obtain a database connection,
     *                     perform rollback operations on the connection in case an error is encountered or commit the
     *                     connection in case the operation has succeeded. Furthermore this instance does retrieve the
     *                     document instances from the persistent storage
     * @param collectionId identifier of document collection index to retrieve
     *
     * @return CollectionIndex with specified identifier or <tt>null</tt> if the collection index with the given
     *         identifier was not found in persistence
     *
     * @throws IllegalArgumentException  when collectionId is <tt>null</tt> or empty (trim'd) string or persistence is
     *                                   <tt>null</tt>
     * @throws IndexPersistenceException when collection index can not be retrieved
     */
    static CollectionIndex getCollectionIndex(final AbstractDBIndexPersistence persistence, final String collectionId)
        throws IndexPersistenceException {
        if (collectionId == null) {
            throw new IllegalArgumentException("The parameter named [collectionId] was null.");
        }
        if (collectionId.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [collectionId] was an empty String.");
        }
        if (persistence == null) {
            throw new IllegalArgumentException("The parameter named [persistence] was null.");
        }

        final Object[] collectionIdQueryArgs = new Object[]{collectionId};
        final Connection connection = persistence.getConnection();

        //check whether collection exists, return null if not
        final Integer count = (Integer) DatabaseUtils.doSingleValueQuery(connection, persistence,
            "SELECT COUNT(*) FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", collectionIdQueryArgs,
            DatabaseUtils.INTEGER_TYPE);
        if (count.intValue() == 0) {
            DatabaseUtils.closeSilently(connection);
            return null;
        }

        final Map wordsOfCollection = new HashMap();
        final Set documentsOfCollection = new HashSet();

        final List documentIds = DatabaseUtils.doSingleColumnQuery(connection, persistence,
            "SELECT DOCUMENT_ID FROM DOCUMENT_COLLECTION_DOCUMENT_XREF where COLLECTION_ID=?",
            collectionIdQueryArgs, DatabaseUtils.STRING_TYPE);
        for (Iterator iterator = documentIds.iterator(); iterator.hasNext();) {
            final String documentId = (String) iterator.next();
            final WordSourceId sourceId = getWordSourceId(persistence, connection, documentId);
            final DocumentIndex index = persistence.getDocumentIndex(sourceId);

            // add all words of the index to the collection's word-index
            // map and add a reference to the current index
            final Set wordsOfIndex = index.getWords().keySet();
            for (Iterator iterator2 = wordsOfIndex.iterator(); iterator2.hasNext();) {
                final CollationKey collationKey = (CollationKey) iterator2.next();
                Set documentsCotainingWord = (Set) wordsOfCollection.get(collationKey);
                if (documentsCotainingWord == null) {
                    documentsCotainingWord = new HashSet();
                    wordsOfCollection.put(collationKey, documentsCotainingWord);
                }
                documentsCotainingWord.add(sourceId);
            }
            // add document to collection's contained documents set
            documentsOfCollection.add(sourceId);
        }
        // build the collection object and return it
        DatabaseUtils.closeSilently(connection);
        return new CollectionIndex(null, wordsOfCollection, documentsOfCollection, collectionId);
    }

    /**
     * This method does remove the document collection index with specified identifier from the persistence.
     * <p/>
     * CS section 1.4.3.4 describes the algorithm of this method.
     *
     * @param collectionId Identifier of document collection index to remove
     * @param persistence  the AbstractDBIndexPersistence instance that can be used to obtain a database connection,
     *                     perform rollback operations on the connection in case an error is encountered or commit the
     *                     connection in case the operation has succeeded
     *
     * @throws IllegalArgumentException  when collectionId is <tt>null</tt> or empty (trim'd) string or persistence is
     *                                   <tt>null</tt>
     * @throws IndexPersistenceException when fails to remove CollectionIndex with given identifier from database, or
     *                                   when collection with that specific identifier did not exist in database
     */
    static void removeCollectionIndex(final AbstractDBIndexPersistence persistence, final String collectionId)
        throws IndexPersistenceException {
        if (collectionId == null) {
            throw new IllegalArgumentException("The parameter named [collectionId] was null.");
        }
        if (collectionId.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [collectionId] was an empty String.");
        }
        if (persistence == null) {
            throw new IllegalArgumentException("The parameter named [persistence] was null.");
        }

        final Connection connection = persistence.getConnection();
        persistence.beginTransaction(connection);

        final Object[] queryArgsCollectionId = new Object[]{collectionId};
        DatabaseUtils.doDMLQuery(connection, persistence,
            "DELETE FROM DOCUMENT_COLLECTION_DOCUMENT_XREF WHERE COLLECTION_ID=?", queryArgsCollectionId);
        final int updateCount = DatabaseUtils.doDMLQuery(connection, persistence,
            "DELETE FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", queryArgsCollectionId);
        if (updateCount == 0) {
            persistence.rollbackTransaction(connection);
            throw new IndexPersistenceException(
                "The collection with the given id [" + collectionId + "] did not exist in the database.");
        }
        persistence.commitTransaction(connection);
        //counts are not updated as indexer implementation is responsible for that
    }

    /**
     * This method does update specified CollectionIndex in the persistence.
     * <p/>
     * CS section 1.4.3.3 describes the algorithm of this method.
     *
     * @param collectionIndex CollectionIndex to update
     * @param persistence     the AbstractDBIndexPersistence instance that can be used to obtain a database connection,
     *                        perform rollback operations on the connection in case an error is encountered or commit
     *                        the connection in case the operation has succeeded
     *
     * @throws IllegalArgumentException  if any parameter is <tt>null</tt>
     * @throws IndexPersistenceException if any error happens when updating the collection index in the persistence,
     *                                   this includes the case, when the specified collectionIndex is not found in the
     *                                   persistence
     */
    static void updateCollectionIndex(final AbstractDBIndexPersistence persistence,
                                      final CollectionIndex collectionIndex) throws
        IndexPersistenceException {
        if (collectionIndex == null) {
            throw new IllegalArgumentException("The parameter named [collectionIndex] was null.");
        }
        if (persistence == null) {
            throw new IllegalArgumentException("The parameter named [persistence] was null.");
        }

        final String collectionId = collectionIndex.getId();
        final Object[] collectionIdQueryArgs = new Object[]{collectionId};
        final Connection connection = persistence.getConnection();

        //check whether collection exists, throw exception if not
        final Integer count = (Integer) DatabaseUtils.doSingleValueQuery(connection, persistence,
            "SELECT COUNT(*) FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", collectionIdQueryArgs,
            DatabaseUtils.INTEGER_TYPE);
        if (count.intValue() == 0) {
            DatabaseUtils.closeSilently(connection);
            throw new IndexPersistenceException(
                "The given collection with the id [" + collectionId + "] did not exist in the persistence.");
        }

        //delete all currently existent records
        DatabaseUtils.doDMLQuery(connection, persistence,
            "DELETE FROM DOCUMENT_COLLECTION_DOCUMENT_XREF WHERE COLLECTION_ID=?", collectionIdQueryArgs);

        // newly insert all records
        final Set allDocumentIds = collectionIndex.getAllDocumentIds();
        for (Iterator iterator = allDocumentIds.iterator(); iterator.hasNext();) {
            final WordSourceId sourceId = (WordSourceId) iterator.next();
            final String documentId = Utils.createIdString(sourceId);
            DatabaseUtils.doDMLQuery(connection, persistence,
                "INSERT INTO DOCUMENT_COLLECTION_DOCUMENT_XREF (COLLECTION_ID,DOCUMENT_ID) VALUES (?,?)",
                new Object[]{collectionId, documentId});
            // Use count is maintained externally
        }
        persistence.commitTransaction(connection);
    }

    /**
     * This method increases the use count value for document index with specified WordSourceId by the given amount.
     *
     * @param persistence  the AbstractDBIndexPersistence instance that can be used to obtain a database connection,
     *                     perform rollback operations on the connection in case an error is encountered or commit the
     *                     connection in case the operation has succeeded
     * @param wordSourceId WordSourceId of document index of which to update use count
     * @param amount       the amount by which to increase the document use count, may also be a negative integer, then
     *                     the result is a decreased use count
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to increase document index use count in the persistence or the
     *                                   document with the given wordSourceId does not exist in persistence
     */
    static void increaseDocumentUseCount(final AbstractDBIndexPersistence persistence, final WordSourceId wordSourceId,
                                         final int amount) throws IndexPersistenceException {
        if (wordSourceId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }
        if (persistence == null) {
            throw new IllegalArgumentException("The parameter named [persistence] was null.");
        }

        final String documentId = Utils.createIdString(wordSourceId);
        final Connection connection = persistence.getConnection();
        persistence.beginTransaction(connection);
        final int useCount = getUseCount(persistence, connection, documentId);
        setUseCount(persistence, connection, useCount + amount, documentId);
        persistence.commitTransaction(connection);
    }

    /**
     * This method does return the set of WordSourceId of documents that exist in this persistence instance.
     * <p/>
     * CS section 1.4.1.1 describes the algorithm of this method.
     *
     * @param persistence the AbstractDBIndexPersistence instance that can be used to obtain a database connection,
     *                    perform rollback operations on the connection in case an error is encountered or commit the
     *                    connection in case the operation has succeeded
     *
     * @return set of WordSourceId of documents that exist in this persistence
     *
     * @throws IllegalArgumentException  in case the given persistence is <tt>null</tt>
     * @throws IndexPersistenceException when the retrieval fails
     */
    static Set getIndexedDocuments(final AbstractDBIndexPersistence persistence) throws IndexPersistenceException {
        if (persistence == null) {
            throw new IllegalArgumentException("The parameter named [persistence] was null.");
        }

        final Connection connection = persistence.getConnection();
        final List result = DatabaseUtils.doSingleColumnQuery(connection, persistence,
            "SELECT DOCUMENT_ID FROM DOCUMENT", new Object[0], DatabaseUtils.STRING_TYPE);

        final Set ret = new HashSet(result.size());
        for (Iterator iterator = result.iterator(); iterator.hasNext();) {
            final String documentId = (String) iterator.next();
            ret.add(getWordSourceId(persistence, connection, documentId));
        }
        DatabaseUtils.closeSilently(connection);
        return ret;
    }

    /**
     * This method does return the use count of the document index with the given id.
     *
     * @param persistence  the AbstractDBIndexPersistence instance that can be used to obtain a database connection,
     *                     perform rollback operations on the connection in case an error is encountered or commit the
     *                     connection in case the operation has succeeded
     * @param wordSourceId WordSourceId of document index of which to retrieve use count
     *
     * @return the use count of the document identified by the given id
     *
     * @throws IllegalArgumentException  if any arg is <tt>null</tt>
     * @throws IndexPersistenceException when fails to retrieve document index use count in the persistence or the
     *                                   document with the given id does not exist in the persistence
     */
    static int getDocumentUseCount(final AbstractDBIndexPersistence persistence, final WordSourceId wordSourceId)
        throws IndexPersistenceException {
        if (wordSourceId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }
        if (persistence == null) {
            throw new IllegalArgumentException("The parameter named [persistence] was null.");
        }

        final String documentId = Utils.createIdString(wordSourceId);
        final Connection connection = persistence.getConnection();
        final int useCount = getUseCount(persistence, connection, documentId);
        DatabaseUtils.closeSilently(connection);
        return useCount;
    }

    /**
     * This method does persist the document header data stored in the given WordSourceId into a new record in the
     * DOCUMENT table.
     * <p/>
     * CS section 1.4.1.3 (steps 2 and 3) describes the algorithm of this method.
     *
     * @param persistence the AbstractDBIndexPersistence instance that can be used to perform rollback operations on the
     *                    connection in case an error is encountered
     * @param sourceId    the source identity representing the document to be persisted
     * @param connection  the connection on which to perform the database operation
     *
     * @return the calculated id string representing the given wordSourceId as database key
     *
     * @throws IllegalArgumentException in case any arg is <tt>null</tt>
     * @throws com.topcoder.document.index.persistence.IndexPersistenceException
     *                                  when fails to add DocumentIndex to the persistence or given index is already
     *                                  persisted
     */

    static String insertDocumentHeaderData(final AbstractDBIndexPersistence persistence, final WordSourceId sourceId,
                                           final Connection connection) throws
        IndexPersistenceException {
        if (persistence == null) {
            throw new IllegalArgumentException("The parameter named [persistence] was null.");
        }
        if (sourceId == null) {
            throw new IllegalArgumentException("The parameter named [sourceId] was null.");
        }
        if (connection == null) {
            throw new IllegalArgumentException("The parameter named [connection] was null.");
        }

        // the validity of these values (CS 1.4.1.3) is guaranteed by the final class
        // WordSourceId and its constructor logic
        final Locale locale = sourceId.getSourceLocale();
        final String[] delimiters = sourceId.getDelimiters();

        final String documentId = Utils.createIdString(sourceId);
        final byte[] sourceIdentityBinary = Utils.serializeObject(sourceId.getSourceIdentity());
        final Integer count = (Integer) DatabaseUtils.doSingleValueQuery(connection, persistence,
            "SELECT COUNT(*) from DOCUMENT WHERE DOCUMENT_ID=?",
            new Object[]{documentId}, DatabaseUtils.INTEGER_TYPE);
        if (count.intValue() > 0) {
            throw new IndexPersistenceException("The given document index does already exist in the database.");
        }
        // create the DOCUMENT record
        DatabaseUtils.doDMLQuery(connection, persistence,
            "INSERT INTO DOCUMENT (DOCUMENT_ID,LOCALE,SOURCE_IDENTITY,CLASS_NAME,USE_COUNT) VALUES (?,?,?,?,?)",
            new Object[]{
                documentId,
                locale.toString(),
                sourceIdentityBinary,
                sourceId.getSourceClassName(),
                new Integer(0)
            });

        // insert the DELIMITERs
        for (int i = 0; i < delimiters.length; i++) {

            DatabaseUtils.doDMLQuery(connection, persistence,
                "INSERT INTO DELIMITER (DOCUMENT_ID,DELIMITER_ENTRY,DELIMITER_SORT_ORDER) VALUES (?,?,?)",
                new Object[]{
                    documentId,
                    delimiters[i],
                    new Integer(i)
                });
        }
        return documentId;
    }

    /**
     * This method sets the use count value for document index with specified documentId to the given value.
     *
     * @param persistence the AbstractDBIndexPersistence instance that can be used to obtain a database connection,
     *                    perform rollback operations on the connection in case an error is encountered or commit the
     *                    connection in case the operation has succeeded
     * @param connection  the connection on which to operate
     * @param documentId  id of document index of which to update use count
     * @param newUseCount new use count value
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to increase document index use count in the persistence or the
     *                                   document with the given wordSourceId does not exist in persistence
     */
    private static void setUseCount(final AbstractDBIndexPersistence persistence, final Connection connection,
                                    final int newUseCount,
                                    final String documentId) throws IndexPersistenceException {
        DatabaseUtils.doDMLQuery(connection, persistence, "UPDATE DOCUMENT SET USE_COUNT=? WHERE DOCUMENT_ID=?",
            new Object[]{new Integer(newUseCount), documentId});
    }

    /**
     * This method retrieves the use count value for document index with specified WordSourceId.
     *
     * @param persistence the AbstractDBIndexPersistence instance that can be used to obtain a database connection,
     *                    perform rollback operations on the connection in case an error is encountered or commit the
     *                    connection in case the operation has succeeded
     * @param connection  the connection on which to operate
     * @param documentId  id of document index of which to retrieve use count
     *
     * @return the use count of the document with the given id
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to increase document index use count in the persistence or the
     *                                   document with the given wordSourceId does not exist in persistence
     */
    private static int getUseCount(final AbstractDBIndexPersistence persistence, final Connection connection,
                                   final String documentId)
        throws IndexPersistenceException {
        //check whether document exists, throw exception if not
        final List result = DatabaseUtils.doQuery(connection, persistence,
            "SELECT USE_COUNT FROM DOCUMENT WHERE DOCUMENT_ID=?",
            new Object[]{documentId},
            new DatabaseUtils.DataType[]{DatabaseUtils.INTEGER_TYPE});
        if (result.isEmpty()) {
            DatabaseUtils.closeSilently(connection);
            throw new IndexPersistenceException(
                "The given document with the database id [" + documentId + "] did not exist in the persistence.");
        }

        return ((Integer) ((Object[]) result.get(0))[0]).intValue();
    }

}
