/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.document.index.persistence.IndexPersistenceException;

import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * This class contains utility methods used only by the database package of this component. This class is
 * package-visible as all methods are solely used from this package.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
class DatabaseUtils {
    /**
     * This constant provides the {@link DataType} instance that can be used in {@link #doQuery(java.sql.Connection,
     * String, Object[], DatabaseUtils.DataType[])} and {@link #doSingleValueQuery(java.sql.Connection, String,
     * Object[], DatabaseUtils.DataType)} to specify that a {@link java.sql.ResultSet} column of a query result should
     * be returned as a non-<tt>null</tt> value of type {@link String}.
     * <p/>
     * The query execution throws an exception case the {@link java.sql.ResultSet} value was <tt>null</tt>.
     */
    static final DataType STRING_TYPE = new StringType();
    /**
     * This constant provides the {@link DataType} instance that can be used in {@link #doQuery(java.sql.Connection,
     * String, Object[], DatabaseUtils.DataType[])} and {@link #doSingleValueQuery(java.sql.Connection, String,
     * Object[], DatabaseUtils.DataType)} to specify that a {@link java.sql.ResultSet} column of a query result should
     * be returned as a non-<tt>null</tt> value of type <tt>byte[]</tt>
     * <p/>
     * The query execution throws an exception case the {@link java.sql.ResultSet} value was <tt>null</tt>.
     */
    static final DataType BYTE_ARRAY_TYPE = new ByteArrayType();
    /**
     * This constant provides the {@link DataType} instance that can be used in {@link #doQuery(java.sql.Connection,
     * String, Object[], DatabaseUtils.DataType[])} and {@link #doSingleValueQuery(java.sql.Connection, String,
     * Object[], DatabaseUtils.DataType)} to specify that a {@link ResultSet} column of a query result should be
     * returned as non-<tt>null</tt> value of type {@link Integer}.
     * <p/>
     * The query execution throws an exception case the {@link java.sql.ResultSet} value was <tt>null</tt>.
     */
    static final DataType INTEGER_TYPE = new IntegerType();

    /**
     * As this class is a utility class, a private constructor is added to avoid class instantiation.
     */
    private DatabaseUtils() {
    }

    /**
     * This method performs the given retrieval (i.e. non-DML) query on the given connection using the given query
     * arguments. The {@link java.sql.ResultSet} returned from the query is fetched into a List of <tt>Object[]</tt>s
     * and then returned. This approach assured that all resources (the {@link java.sql.PreparedStatement} and the
     * {@link java.sql.ResultSet}) allocated in this method are also de-allocated in this method.
     * <p/>
     * <b>Note:</b> The given connection is not closed or committed in this method, but is rolled back upon any error
     *
     * @param connection  the connection to perform the query on
     * @param persistence the AbstractDBIndexPersistence instance that can be used to perform rollback operations on the
     *                    given connection in case an error is encountered
     * @param queryString the query to be performed (a {@link java.sql.PreparedStatement} is created from it)
     * @param queryArgs   the arguments to be used in the query (i.e. to be set to the created {@link
     *                    PreparedStatement})
     * @param columnTypes the types as which to return the result set columns, use the values {@link #STRING_TYPE},
     *                    {@link #BYTE_ARRAY_TYPE} or {@link #INTEGER_TYPE} in the construction of the given array
     *
     * @return the result of the query as List containing an <tt>Object[]</tt> for each {@link java.sql.ResultSet} row
     *         The elements of the array are of the type represented by the {@link DataType} specified at the
     *         corresponding index in the given columnTypes array (or <tt>null</tt> in case the resultSet value was
     *         <tt>null</tt>)
     *
     * @throws IndexPersistenceException in case the query fails
     * @throws IllegalArgumentException  in case any arg is <tt>null</tt> or the given query String was an empty
     *                                   (trim'd) string or the column types count does not match the result set column
     *                                   count or column types contained a <tt>null</tt> value
     * @see DatabaseUtils#doDMLQuery(java.sql.Connection, AbstractDBIndexPersistence, String, Object[])
     * @see DatabaseUtils#doSingleValueQuery(Connection, AbstractDBIndexPersistence, String, Object[],
     *      DatabaseUtils.DataType)
     */
    static List doQuery(final Connection connection, final AbstractDBIndexPersistence persistence,
                        final String queryString, final Object[] queryArgs, final DataType[] columnTypes)
        throws IndexPersistenceException {
        DatabaseUtils.checkArgs(connection, persistence, queryString, queryArgs);
        if (columnTypes == null) {
            throw new IllegalArgumentException("The parameter named [columnTypes] was null.");
        }
        for (int i = 0; i < columnTypes.length; i++) {
            if (columnTypes[i] == null) {
                throw new IllegalArgumentException("The given parameter named [columnTypes] contained a null value.");
            }
        }

        try {
            final PreparedStatement preparedStatement = connection.prepareStatement(queryString);
            try {
                for (int i = 0; i < queryArgs.length; i++) {
                    preparedStatement.setObject(i + 1, queryArgs[i]);
                }
                final ResultSet resultSet = preparedStatement.executeQuery();
                try {
                    final List ret = new ArrayList();
                    final ResultSetMetaData metaData = resultSet.getMetaData();
                    final int columnCount = metaData.getColumnCount();
                    if (columnTypes.length != columnCount) {
                        throw new IllegalArgumentException("The column types length [" + columnTypes.length
                            + "] does not match the result set column count[" + columnCount + "].");
                    }
                    while (resultSet.next()) {
                        final Object[] rowData = new Object[columnCount];
                        for (int i = 0; i < rowData.length; i++) {
                            rowData[i] = columnTypes[i].getValue(resultSet, i + 1);
                        }
                        ret.add(rowData);
                    }
                    return ret;
                } finally {
                    DatabaseUtils.closeResultSetSilently(resultSet);
                }
            } finally {
                DatabaseUtils.closeStatementSilently(preparedStatement);
            }

        } catch (SQLException e) {
            // this is the catch clause for exceptions that can occur during execution of the statement,
            // Upon exception we rollback and close the connection and throw the wrapped SQLException
            persistence.rollbackTransaction(connection);
            throw new IndexPersistenceException(
                "Error while executing query [" + queryString + "] using the query arguments [" + Arrays.asList(
                    queryArgs).toString() + "].", e);
        }
    }

    /**
     * This method performs the given retrieval (i.e. non-DML) query on the given connection using the given query
     * arguments.
     * <p/>
     * The {@link java.sql.ResultSet} returned from the query is expected to contain ONE row containing ONE value, which
     * is then returned.
     * <p/>
     * This approach assured that all resources (the {@link java.sql.PreparedStatement} and the {@link
     * java.sql.ResultSet}) allocated in this method are also de-allocated in this method.
     * <p/>
     * <b>Note:</b> The given connection is not closed or committed in this method, but is rolled back upon any error
     *
     * @param connection  the connection to perform the query on
     * @param persistence the AbstractDBIndexPersistence instance that can be used to perform rollback operations on the
     *                    given connection in case an error is encountered
     * @param queryString the query to be performed (a {@link java.sql.PreparedStatement} is created from it)
     * @param queryArgs   the arguments to be used in the query (i.e. to be set to the created {@link
     *                    PreparedStatement})
     * @param columnType  the type as which to return the value, use one of the values {@link #STRING_TYPE}, {@link
     *                    #BYTE_ARRAY_TYPE} or {@link #INTEGER_TYPE} here
     *
     * @return the value returned by the query as value of the type represented by the given columnType or <tt>null</tt>
     *         if the {@link java.sql.ResultSet} value was <tt>null</tt>
     *
     * @throws IndexPersistenceException in case the query fails
     * @throws IllegalArgumentException  in case any arg is <tt>null</tt> or the given query String was an empty
     *                                   (trim'd) string or the given query did return multiple rows or columns
     * @see DatabaseUtils#doDMLQuery(java.sql.Connection, AbstractDBIndexPersistence, String, Object[])
     * @see DatabaseUtils#doSingleValueQuery(java.sql.Connection, AbstractDBIndexPersistence, String, Object[],
     *      DatabaseUtils.DataType)
     */
    static Object doSingleValueQuery(final Connection connection, final AbstractDBIndexPersistence persistence,
                                     final String queryString, final Object[] queryArgs,
                                     final DataType columnType)
        throws IndexPersistenceException {
        DatabaseUtils.checkArgs(connection, persistence, queryString, queryArgs);
        if (columnType == null) {
            throw new IllegalArgumentException("The parameter named [columnType] was null.");
        }

        try {
            final PreparedStatement preparedStatement = connection.prepareStatement(queryString);
            try {
                for (int i = 0; i < queryArgs.length; i++) {
                    preparedStatement.setObject(i + 1, queryArgs[i]);
                }
                final ResultSet resultSet = preparedStatement.executeQuery();
                try {
                    final ResultSetMetaData metaData = resultSet.getMetaData();
                    final int columnCount = metaData.getColumnCount();
                    if (columnCount == 1 && resultSet.next()) {
                        return columnType.getValue(resultSet, 1);
                    } else {
                        throw new IllegalArgumentException(
                            "The given query [" + queryString + "] did not return ONE result row containing "
                                + "ONE value using the query arguments [" + Arrays.asList(queryArgs).toString() + "].");
                    }
                } finally {
                    DatabaseUtils.closeResultSetSilently(resultSet);
                }
            } finally {
                DatabaseUtils.closeStatementSilently(preparedStatement);
            }

        } catch (SQLException e) {
            // this is the catch clause for exceptions that can occur during execution of the statement,
            // Upon exception we rollback and close the connection and throw the wrapped SQLException
            //
            // The rollback is done even as this is no DML query,
            // as previous queries on this connection could have been DMLs
            persistence.rollbackTransaction(connection);
            throw new IndexPersistenceException(
                "Error while executing query [" + queryString + "] using the query arguments [" + Arrays.asList(
                    queryArgs).toString() + "].", e);
        }
    }

    /**
     * This method performs the given retrieval (i.e. non-DML) query on the given connection using the given query
     * arguments.
     * <p/>
     * The {@link java.sql.ResultSet} returned from the query is expected to contain any number of rows but exactly ONE
     * column, Sot hat the returned list contains the result values. In comparison to {@link
     * #doQuery(java.sql.Connection, AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])} this
     * method does return a list that directly contains the values and does not wrap every value in an Object[]
     * containing only the value.
     * <p/>
     * This approach assured that all resources (the {@link java.sql.PreparedStatement} and the {@link
     * java.sql.ResultSet}) allocated in this method are also de-allocated in this method.
     * <p/>
     * <b>Note:</b> The given connection is not closed or committed in this method, but is rolled back upon any error
     *
     * @param connection  the connection to perform the query on
     * @param persistence the AbstractDBIndexPersistence instance that can be used to perform rollback operations on the
     *                    given connection in case an error is encountered
     * @param queryString the query to be performed (a {@link java.sql.PreparedStatement} is created from it)
     * @param queryArgs   the arguments to be used in the query (i.e. to be set to the created {@link
     *                    PreparedStatement})
     * @param columnType  the type as which to return the value, use one of the values {@link #STRING_TYPE}, {@link
     *                    #BYTE_ARRAY_TYPE} or {@link #INTEGER_TYPE} here
     *
     * @return the values returned by the query as List of values of the type represented by the given columnType
     *
     * @throws IndexPersistenceException in case the query fails
     * @throws IllegalArgumentException  in case any arg is <tt>null</tt> or the given query String was an empty
     *                                   (trim'd) string or the given query did return multiple rows or columns
     * @see DatabaseUtils#doDMLQuery(java.sql.Connection, AbstractDBIndexPersistence, String, Object[])
     * @see DatabaseUtils#doSingleValueQuery(java.sql.Connection, AbstractDBIndexPersistence, String, Object[],
     *      DatabaseUtils.DataType)
     */
    static List doSingleColumnQuery(final Connection connection, final AbstractDBIndexPersistence persistence,
                                    final String queryString, final Object[] queryArgs,
                                    final DataType columnType)
        throws IndexPersistenceException {
        DatabaseUtils.checkArgs(connection, persistence, queryString, queryArgs);
        if (columnType == null) {
            throw new IllegalArgumentException("The parameter named [columnType] was null.");
        }

        try {
            final PreparedStatement preparedStatement = connection.prepareStatement(queryString);
            try {
                for (int i = 0; i < queryArgs.length; i++) {
                    preparedStatement.setObject(i + 1, queryArgs[i]);
                }
                final ResultSet resultSet = preparedStatement.executeQuery();
                try {
                    final ResultSetMetaData metaData = resultSet.getMetaData();
                    final int columnCount = metaData.getColumnCount();
                    if (columnCount == 1) {
                        final List ret = new ArrayList();
                        while (resultSet.next()) {
                            ret.add(columnType.getValue(resultSet, 1));
                        }
                        return ret;
                    } else {
                        throw new IllegalArgumentException(
                            "The given query [" + queryString + "] did not return "
                                + "ONE column using the query arguments [" + Arrays.asList(queryArgs).toString()
                                + "].");
                    }
                } finally {
                    DatabaseUtils.closeResultSetSilently(resultSet);
                }
            } finally {
                DatabaseUtils.closeStatementSilently(preparedStatement);
            }

        } catch (SQLException e) {
            // this is the catch clause for exceptions that can occur during execution of the statement,
            // Upon exception we rollback and close the connection and throw the wrapped SQLException
            //
            // The rollback is done even as this is no DML query,
            // as previous queries on this connection could have been DMLs
            persistence.rollbackTransaction(connection);
            throw new IndexPersistenceException(
                "Error while executing query [" + queryString + "] using the query arguments [" + Arrays.asList(
                    queryArgs).toString() + "].", e);
        }
    }

    /**
     * This method performs the given DML (query on the given connection using the given query arguments.
     * <p/>
     * The update count returned from the query is then returned.
     * <p/>
     * This approach assured that all resources (the {@link java.sql.PreparedStatement}) allocated in this method are
     * also de-allocated in this method.
     * <p/>
     * <b>Note:</b> The given connection is not closed or committed in this method, but is rolled back upon any error
     *
     * @param connection  the connection to perform the query on
     * @param persistence the AbstractDBIndexPersistence instance that can be used to perform rollback operations on the
     *                    given connection in case an error is encountered
     * @param queryString the query to be performed (a {@link java.sql.PreparedStatement} is created from it)
     * @param queryArgs   the arguments to be used in the query (i.e. to be set to the created {@link
     *                    PreparedStatement})
     *
     * @return the number of database rows affected by the query
     *
     * @throws IndexPersistenceException in case the query fails
     * @throws IllegalArgumentException  in case any arg is <tt>null</tt> or the given query String was an empty
     *                                   (trim'd) string
     * @see DatabaseUtils#doQuery(java.sql.Connection, AbstractDBIndexPersistence, String, Object[],
     *      DatabaseUtils.DataType[])
     * @see DatabaseUtils#doSingleValueQuery(java.sql.Connection, AbstractDBIndexPersistence, String, Object[],
     *      DatabaseUtils.DataType)
     */
    static int doDMLQuery(final Connection connection, final AbstractDBIndexPersistence persistence,
                          final String queryString, final Object[] queryArgs) throws IndexPersistenceException {
        DatabaseUtils.checkArgs(connection, persistence, queryString, queryArgs);
        try {
            final PreparedStatement preparedStatement = connection.prepareStatement(queryString);
            try {
                for (int i = 0; i < queryArgs.length; i++) {
                    preparedStatement.setObject(i + 1, queryArgs[i]);
                }
                preparedStatement.execute();
                return preparedStatement.getUpdateCount();
            } finally {
                DatabaseUtils.closeStatementSilently(preparedStatement);
            }
        } catch (SQLException e) {
            // this is the catch clause for exceptions that can occur during execution of the statement,
            // Upon exception we rollback and close the connection and throw the wrapped SQLException
            persistence.rollbackTransaction(connection);
            throw new IndexPersistenceException(
                "Error while executing query [" + queryString + "] using the query arguments [" + Arrays.asList(
                    queryArgs).toString() + "].", e);
        }
    }

    /**
     * This is a utility arg-checking method for the given args.
     *
     * @param connection  checked to be non-<tt>null</tt>
     * @param persistence checked to be non-<tt>null</tt>
     * @param queryString checked to be non-<tt>null</tt> and non-empty(trim'd)
     * @param queryArgs   checked to be non-<tt>null</tt>
     *
     * @throws IllegalArgumentException in case any arg is <tt>null</tt> or the given query String was an empty (trim'd)
     *                                  string
     */
    private static void checkArgs(final Connection connection, final AbstractDBIndexPersistence persistence,
                                  final String queryString, final Object[] queryArgs) {
        if (connection == null) {
            throw new IllegalArgumentException("The parameter named [connection] was null.");
        }
        if (persistence == null) {
            throw new IllegalArgumentException("The parameter named [persistence] was null.");
        }
        if (queryString == null) {
            throw new IllegalArgumentException("The parameter named [queryString] was null.");
        }
        if (queryString.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [queryString] was an empty String.");
        }
        if (queryArgs == null) {
            throw new IllegalArgumentException("The parameter named [queryArgs] was null.");
        }
    }

    /**
     * This method silently closes the given connection.
     * <p/>
     * Silently means that any exceptions thrown during close of the connection are caught and logged but not
     * re-thrown.
     *
     * @param connection the connection to be closed
     *
     * @throws IllegalArgumentException in case the given connection is <tt>null</tt>
     */
    static void closeSilently(final Connection connection) {
        if (connection == null) {
            throw new IllegalArgumentException("The parameter named [connection] was null.");
        }

        try {
            connection.close();
        } catch (SQLException e) {
            // as we do not want this exception to mask an
            // exception that might have occurred during execution, we
            // cannot throw this exception here.
            // Furthermore as this is already a cleanup action which
            // is performed after the operation has finished (either
            // successfully, then the expected result exists or
            // unsuccessfully then already an SQLException exists)
            // there is no reason to throw an exception from this point.
        }
    }

    /**
     * This method closes the given result set silently, i.e. any exception occurring during close is not re-thrown but
     * only logged.
     *
     * @param resultSet the resultSet to be closed
     *
     * @throws IllegalArgumentException in case the given arg is <tt>null</tt>
     */
    private static void closeResultSetSilently(final ResultSet resultSet) {
        if (resultSet == null) {
            throw new IllegalArgumentException("The parameter named [resultSet] was null.");
        }

        try {
            // safely close the result set
            // according to http://jdj.sys-con.com/read/46653.htm
            // a robust application should not rely on implicit
            // closing of result sets
            resultSet.close();
        } catch (SQLException e) {
            // as we do not want this exception to mask an
            // exception that might have occurred during execution, we
            // cannot throw this exception here.
            // Furthermore as this is already a cleanup action which
            // is performed after the operation has finished (either
            // successfully, then the expected result exists or
            // unsuccessfully then already an SQLException exists)
            // there is no reason to throw an exception from this point.
        }
    }

    /**
     * This method closes the given Statement silently, i.e. any exception that occurs during close is not, re.thrown
     * but only logged.
     *
     * @param preparedStatement the statement to be closed
     *
     * @throws IllegalArgumentException in case the given arg is <tt>null</tt>
     */
    private static void closeStatementSilently(final PreparedStatement preparedStatement) {
        if (preparedStatement == null) {
            throw new IllegalArgumentException("The parameter named [preparedStatement] was null.");
        }

        try {
            // safely close the statement
            // according to http://jdj.sys-con.com/read/46653.htm
            // a robust application should not rely on implicit
            // closing of statements
            preparedStatement.close();
        } catch (SQLException e) {
            // as we do not want this exception to mask an
            // exception that might have occurred during execution, we
            // cannot throw this exception here.
            // Furthermore as this is already a cleanup action which
            // is performed after the operation has finished (either
            // successfully, then the expected result exists or
            // unsuccessfully then already an SQLException exists)
            // there is no reason to throw an exception from this point.
        }
    }

    /**
     * This class is a wrapper for type safe retrieval of values from a {@link java.sql.ResultSet}. Instances of this
     * class can be used as arguments to {@link DatabaseUtils#doQuery(java.sql.Connection, AbstractDBIndexPersistence,
     * String, Object[], DatabaseUtils.DataType[])} and {@link DatabaseUtils#doSingleValueQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}to specify the type as which to return the
     * value of a ResultSet column.
     * <p/>
     * This class has been introduced to generify the behaviors of different databases and JDBC drivers so that always
     * the expected type is returned ({@link java.sql.ResultSet#getObject(int)} does not sufficiently do this job as the
     * type of the value is highly database-dependant (e.g. for a BLOB column the MySQL driver returns a <tt>byte[]</tt>
     * and the Oracle driver returns a {@link java.sql.Blob})).
     * <p/>
     * This class contains a private constructor to make sure all implementations of this class are declared inside
     * {@link DatabaseUtils}. Instances are provided to users via constants declared in {@link DatabaseUtils} - so this
     * class defines some kind of 'pseudo-enum' which cannot be instantiated externally.
     *
     * @author TCSDEVELOPER
     * @version 1.0
     */
    abstract static class DataType {
        /**
         * Empty private constructor. By using this concept, it is assured that only {@link DatabaseUtils} can contain
         * subclasses of this class and the implementation classes cannot be instantiated externally.
         */
        private DataType() {

        }

        /**
         * This method retrieves the value at the given index from the given resultSet as instance of the
         * subclass-dependant type.
         * <p/>
         *
         * @param resultSet the result set from which to retrieve the value
         * @param index     the index at which to retrieve the value
         *
         * @return the retrieved value
         *
         * @throws IllegalArgumentException in case given resultSet or columnType is <tt>null</tt>
         * @throws java.sql.SQLException    in case an exception occurs while working with the given ResultSet or the
         *                                  index does not exist in the result set
         */
        protected abstract Object getValue(ResultSet resultSet, int index) throws SQLException;

    }

    /**
     * This class is a wrapper for type safe retrieval of values from a {@link java.sql.ResultSet}. The values retrieved
     * by the {@link #getValue(java.sql.ResultSet, int)} implementation of this {@link DataType} are assured to be of
     * type {@link String} or to be <tt>null</tt> in case the {@link java.sql.ResultSet} value was <tt>null</tt>.
     * <p/>
     * <p/>
     * The only instance of this class is available as value of the constant {@link DatabaseUtils#STRING_TYPE}.
     *
     * @author TCSDEVELOPER
     * @version 1.0
     */
    private static class StringType extends DataType {
        /**
         * This method retrieves the value at the given index from the given resultSet as instance of the
         * subclass-dependant type.
         * <p/>
         *
         * @param resultSet the result set from which to retrieve the value
         * @param index     the index at which to retrieve the value
         *
         * @return the retrieved value as {@link String} or <tt>null</tt> if the value in the {@link java.sql.ResultSet}
         *         was <tt>null</tt>.
         *
         * @throws IllegalArgumentException in case given resultSet or columnType is <tt>null</tt>
         * @throws java.sql.SQLException    in case an exception occurs while working with the given ResultSet or the
         *                                  index does not exist in the result set
         */
        protected Object getValue(final ResultSet resultSet, final int index) throws SQLException {
            final String string = resultSet.getString(index);
            if (resultSet.wasNull()) {
                throw new SQLException("Assertion failed: result set value was expected to be non-null, "
                    + "but was null.");
            } else {
                return string;
            }
        }
    }

    /**
     * This class is a wrapper for type safe retrieval of values from a {@link java.sql.ResultSet}. The values retrieved
     * by the {@link #getValue(java.sql.ResultSet, int)} implementation of this {@link DataType} are assured to be of
     * type <tt>byte[]</tt> or to be <tt>null</tt> in case the {@link java.sql.ResultSet} value was <tt>null</tt>.
     * <p/>
     * <p/>
     * The only instance of this class is available as value of the constant {@link DatabaseUtils#BYTE_ARRAY_TYPE}.
     *
     * @author TCSDEVELOPER
     * @version 1.0
     */
    private static class ByteArrayType extends DataType {
        /**
         * This method retrieves the value at the given index from the given resultSet as instance of the
         * subclass-dependant type.
         * <p/>
         *
         * @param resultSet the result set from which to retrieve the value
         * @param index     the index at which to retrieve the value
         *
         * @return the retrieved value as <tt>byte[]</tt> or <tt>null</tt> if the value in the {@link
         *         java.sql.ResultSet} was <tt>null</tt>.
         *
         * @throws IllegalArgumentException in case given resultSet or columnType is <tt>null</tt>
         * @throws java.sql.SQLException    in case an exception occurs while working with the given ResultSet or the
         *                                  index does not exist in the result set
         */
        protected Object getValue(final ResultSet resultSet, final int index) throws SQLException {
            final Blob blob = resultSet.getBlob(index);
            if (resultSet.wasNull()) {
                throw new SQLException("Assertion failed: result set value was expected to be non-null, "
                    + "but was null.");
            } else {
                return blob.getBytes(1, (int) blob.length());
            }
        }
    }

    /**
     * This class is a wrapper for type safe retrieval of values from a {@link ResultSet}. The values retrieved by the
     * {@link #getValue(java.sql.ResultSet, int)} implementation of this {@link DataType} are assured to be of type
     * {@link Integer} or to be <tt>null</tt> in case the {@link ResultSet} value was <tt>null</tt>.
     * <p/>
     * <p/>
     * The only instance of this class is available as value of the constant {@link DatabaseUtils#INTEGER_TYPE}.
     *
     * @author TCSDEVELOPER
     * @version 1.0
     */
    private static class IntegerType extends DataType {
        /**
         * This method retrieves the value at the given index from the given resultSet as instance of the
         * subclass-dependant type.
         * <p/>
         *
         * @param resultSet the result set from which to retrieve the value
         * @param index     the index at which to retrieve the value
         *
         * @return the retrieved value as {@link Integer} or <tt>null</tt> if the value in the {@link ResultSet} was
         *         <tt>null</tt>.
         *
         * @throws IllegalArgumentException in case given resultSet or columnType is <tt>null</tt>
         * @throws SQLException             in case an exception occurs while working with the given ResultSet or the
         *                                  index does not exist in the result set
         */
        protected Object getValue(final ResultSet resultSet, final int index) throws SQLException {
            final int aInteger = resultSet.getInt(index);
            if (resultSet.wasNull()) {
                throw new SQLException("Assertion failed: result set value was expected to be non-null, "
                    + "but was null.");
            } else {
                return new Integer(aInteger);
            }
        }
    }
}
