/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.DBConnectionFactory;
import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.persistence.impl.Utils;
import com.topcoder.document.index.wordsource.WordSourceId;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.CollationKey;
import java.text.Collator;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;


/**
 * <p>This is an implementation of the AbstractDBIndexPersistence that deals with a fast, CLOB based, normalized data
 * based schema. This is provided for a user how is concerned about speed of the execution. By storing all the word and
 * word position data in a single CLOB structure we are achieving a 4000% speed increase over a comparable solution that
 * uses thousands of records to store words and related positions Not thread-safe</p>
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
public class FastAccessDBIndexPersistence extends AbstractDBIndexPersistence {

    /**
     * This constructor will populate the connectionFactory and connectionName information from configuration.
     *
     * @param namespace The namespace for connection-related properties
     *
     * @throws IllegalArgumentException if namespace param is <tt>null</tt> or an empty (trim'd) String
     * @throws PersistenceConfigurationException
     *                                  if there are configuration issues encountered
     * @see AbstractDBIndexPersistence#AbstractDBIndexPersistence(String)
     */
    public FastAccessDBIndexPersistence(final String namespace) throws PersistenceConfigurationException {
        // arg checking is done in super constructor
        super(namespace);
    }

    /**
     * This constructor will populate the connectionFactory and connectionName information from input parameters.
     *
     * @param connectionFactory DBConnectionFactory instance used to generate connections
     * @param connectionName    The name of the connection used for persistence operations
     *
     * @throws IllegalArgumentException if connectionFactory is <tt>null</tt>
     */
    public FastAccessDBIndexPersistence(final DBConnectionFactory connectionFactory, final String connectionName) {
        // arg checking is done in super constructor
        super(connectionFactory, connectionName);
    }

    /**
     * This method adds the given DocumentIndex to the persistence.
     * <p/>
     * CS section 1.4.2.1 describes the algorithm of this method.
     *
     * @param documentIndex WordSource representing the document to be persisted
     *
     * @throws IllegalArgumentException  when documentIndex is <tt>null</tt>
     * @throws IndexPersistenceException when fails to add DocumentIndex to the persistence or given index is already
     *                                   persisted
     */
    public void addDocumentIndex(final DocumentIndex documentIndex) throws IndexPersistenceException {
        if (documentIndex == null) {
            throw new IllegalArgumentException("The parameter named [documentIndex] was null.");
        }
        final Connection connection = getConnection();
        beginTransaction(connection);

        final String documentId =
            CommonDatabaseOperations.insertDocumentHeaderData(this, documentIndex.getWordSourceId(), connection);

        // the validity of these values (CS 1.4.1.3) is guaranteed by the final class
        // DocumentIndex and its constructor logic
        final Map words = documentIndex.getWords();

        //the delimiter used in the stream to delimit our tokens
        final String posDelim = documentIndex.getWordSourceId().getDelimiters()[0];

        PreparedStatement preparedStatement = null;
        try {
            // This algorithm might seem somewhat cumbersome but it is the only way to obtain a CLOB instance and
            // stream the data to the database in case one wants to avoid the usage of vendor-specific JDBC extensions.
            // general procedure:
            // 1. create record with clob column value being not null,
            // 2. query the clob value,
            // 3. use the clob instance in a new update statement,
            // 4. stream the data into the clob instance and
            // 5. execute the update statement
            preparedStatement = connection.prepareStatement(
                "INSERT INTO WORD_DATA (DOCUMENT_ID,WORD_DATA) VALUES (?,?)");
            preparedStatement.setString(1, documentId);
            preparedStatement.setString(2, "a");
            preparedStatement.executeUpdate();
            preparedStatement.close();
            preparedStatement = connection.prepareStatement("SELECT WORD_DATA FROM WORD_DATA WHERE DOCUMENT_ID=?",
                ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
            preparedStatement.setString(1, documentId);
            final ResultSet resultSet = preparedStatement.executeQuery();
            try {
                resultSet.next();
                final Clob clob = resultSet.getClob(1);

                preparedStatement.close();
                preparedStatement = connection.prepareStatement(
                    "UPDATE WORD_DATA SET WORD_DATA=? WHERE DOCUMENT_ID=?");
                preparedStatement.setClob(1, clob);
                preparedStatement.setString(2, documentId);
                final Writer writer = clob.setCharacterStream(0);

                final Set wordSet = words.entrySet();
                writeWordDataToWriter(wordSet, writer, posDelim);
                preparedStatement.executeUpdate();
            } finally {
                resultSet.close();
            }
        } catch (SQLException e) {
            rollbackTransaction(connection);
            throw new IndexPersistenceException("Error while inserting CLOB data to database", e);
        } catch (IOException e) {
            rollbackTransaction(connection);
            throw new IndexPersistenceException("Error while inserting CLOB data to database", e);
        } finally {
            if (preparedStatement != null) {
                try {
                    preparedStatement.close();
                } catch (SQLException e) {
                    // this exception is minor re-throwing it would either
                    // mask some other exception or break some otherwise successful operation
                }
            }
        }

        commitTransaction(connection);
    }

    /**
     * This method does retrieve a DocumentIndex with the specified WordSourceId from the persistence. It does return
     * <tt>null</tt>, if DocumentIndex with the given WordSourceId is not found in the persistence.
     * <p/>
     * CS section 1.4.1.1 describes the algorithm of this method.
     *
     * @param wordSourceId WordSourceId of document to retrieve
     *
     * @return retrieved DocumentIndex, or <tt>null</tt> if document index with given WordSourceId is not found in the
     *         persistence
     *
     * @throws IllegalArgumentException  when wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to retrieve document index with the specified WordSourceId
     */
    public DocumentIndex getDocumentIndex(final WordSourceId wordSourceId)
        throws IndexPersistenceException {
        if (wordSourceId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }

        final String documentId = Utils.createIdString(wordSourceId);

        final Object[] queryArgsDocumentId = new Object[]{documentId};

        final Connection connection = getConnection();
        final Integer count = (Integer) DatabaseUtils.doSingleValueQuery(connection, this,
            "SELECT COUNT(*) FROM DOCUMENT WHERE DOCUMENT_ID=?", queryArgsDocumentId,
            DatabaseUtils.INTEGER_TYPE);
        if (count.intValue() == 0) {
            DatabaseUtils.closeSilently(connection);
            return null;
        }

        final Locale locale = wordSourceId.getSourceLocale();
        final Collator collator = Collator.getInstance(locale);

        final Map words;
        final String delim = wordSourceId.getDelimiters()[0];

        try {
            final PreparedStatement preparedStatement = connection.prepareStatement(
                "SELECT WORD_DATA FROM WORD_DATA WHERE DOCUMENT_ID=?");
            preparedStatement.setString(1, documentId);
            final ResultSet resultSet = preparedStatement.executeQuery();
            try {
                resultSet.next();
                final Reader characterStream = resultSet.getClob(1).getCharacterStream();
                words = readWordDataFromReader(characterStream, delim, collator);
            } finally {
                resultSet.close();
            }
        } catch (SQLException e) {
            rollbackTransaction(connection);
            throw new IndexPersistenceException("Error while reading CLOB data from database", e);
        } catch (IOException e) {
            rollbackTransaction(connection);
            throw new IndexPersistenceException("Error while reading CLOB data from database", e);
        }
        DatabaseUtils.closeSilently(connection);
        return new DocumentIndex(wordSourceId, words);
    }

    /**
     * This method does remove a document index with given WordSourceId from the persistence. When the document index is
     * not found, its use count is greater that zero, or function fail to remove it exception is thrown.
     * <p/>
     * CS section 1.4.1.2 describes the algorithm of this method.
     *
     * @param wordSourceId WordSourceId of  document index to remove
     *
     * @throws IllegalArgumentException  when WordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when document index is not found in the persistence, its use count is not zero,
     *                                   or error happens when trying to remove it
     */
    public void removeDocumentIndex(final WordSourceId wordSourceId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        if (wordSourceId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }

        final String documentId = Utils.createIdString(wordSourceId);

        final Connection connection = getConnection();
        final Object[] documentIdQueryArg = new Object[]{documentId};
        final int count = getDocumentUseCount(wordSourceId);
        if (count != 0) {
            throw new IndexPersistenceException("The index with the given id [" + wordSourceId
                + "] cannot be deleted as its use count is not 0, but is " + count + ".");
        }

        beginTransaction(connection);

        DatabaseUtils.doDMLQuery(connection, this, "DELETE FROM WORD_DATA WHERE DOCUMENT_ID=?",
            documentIdQueryArg);
        DatabaseUtils.doDMLQuery(connection, this, "DELETE FROM DELIMITER WHERE DOCUMENT_ID=?",
            documentIdQueryArg);
        DatabaseUtils.doDMLQuery(connection, this, "DELETE FROM DOCUMENT WHERE DOCUMENT_ID=?",
            documentIdQueryArg);
        // the DOCUMENT_COLLECTION_DOCUMENT_XREF table remains untouched, as the indexer component assures no
        // matching entry to exist in that table when use count of document is 0

        commitTransaction(connection);
    }

    /**
     * This method does store the given document collection index in the persistence.
     * <p/>
     * CS section 1.4.3.1 describes the algorithm of this method.
     *
     * @param collectionIndex CollectionIndex to store
     *
     * @throws IllegalArgumentException  if collectionIndex is <tt>null</tt>
     * @throws IndexPersistenceException if fails to store document collection index or given index is already
     *                                   persisted
     */
    public void addCollectionIndex(final CollectionIndex collectionIndex)
        throws IndexPersistenceException {
        //arg checking and everything else is in common method
        CommonDatabaseOperations.addCollectionIndex(this, collectionIndex);
    }

    /**
     * This method retrieves a document collection index with specified identifier. It does return <tt>null</tt> when
     * collection with specified identifier is not found.
     * <p/>
     * CS section 1.4.3.2 describes the algorithm of this method.
     *
     * @param collectionId identifier of document collection index to retrieve
     *
     * @return CollectionIndex with specified identifier or <tt>null</tt> if the collection index with the given
     *         identifier was not found in persistence
     *
     * @throws IllegalArgumentException  when collectionId is <tt>null</tt> or an empty (trim'd) string
     * @throws IndexPersistenceException when collection index can not be retrieved
     */
    public CollectionIndex getCollectionIndex(final String collectionId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        return CommonDatabaseOperations.getCollectionIndex(this, collectionId);
    }

    /**
     * This method does remove the document collection index with specified identifier from the persistence.
     * <p/>
     * CS section 1.4.3.4 describes the algorithm of this method.
     *
     * @param collectionId identifier of document collection index to remove
     *
     * @throws IllegalArgumentException  when collectionId is <tt>null</tt> or empty (trim'd) string
     * @throws IndexPersistenceException when fails to remove CollectionIndex with given identifier from database, or
     *                                   when collection with that specific identifier did not exist in database
     */
    public void removeCollectionIndex(final String collectionId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        CommonDatabaseOperations.removeCollectionIndex(this, collectionId);
    }

    /**
     * This method does update specified CollectionIndex in the persistence.
     * <p/>
     * CS section 1.4.3.3 describes the algorithm of this method.
     *
     * @param collectionIndex CollectionIndex to update
     *
     * @throws IllegalArgumentException  if collectionIndex parameter is <tt>null</tt>
     * @throws IndexPersistenceException if any error happens when updating the collection index in the persistence,
     *                                   this includes the case, when the specified collectionIndex is not found in the
     *                                   persistence
     */
    public void updateCollectionIndex(final CollectionIndex collectionIndex) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        CommonDatabaseOperations.updateCollectionIndex(this, collectionIndex);
    }

    /**
     * This method increases the use count value for document index with specified WordSourceId by 1.
     *
     * @param wordSourceId WordSourceId of document index of which to update use count
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to increase document index use count in the persistence or the
     *                                   document with the given wordSourceId does not exist in persistence
     */
    public void increaseDocumentUseCount(final WordSourceId wordSourceId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        CommonDatabaseOperations.increaseDocumentUseCount(this, wordSourceId, 1);
    }

    /**
     * This method decreases the use count value for document index with specified WordSourceId by 1.
     *
     * @param wordSourceId WordSourceId of document index of which to update use count
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to increase document index use count in the persistence or the
     *                                   document with the given wordSourceId does not exist in persistence
     */
    public void decreaseDocumentUseCount(final WordSourceId wordSourceId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        CommonDatabaseOperations.increaseDocumentUseCount(this, wordSourceId, -1);
    }

    /**
     * This method does return the use count of the document index with the given id.
     *
     * @param wordSourceId WordSourceId of document index of which to retrieve use count
     *
     * @return the use count of the document with the given id
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to retrieve document index use count in the persistence or the
     *                                   document with the given id does not exist in the persistence
     */
    public int getDocumentUseCount(final WordSourceId wordSourceId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        return CommonDatabaseOperations.getDocumentUseCount(this, wordSourceId);
    }

    /**
     * This method does return the set of WordSourceId of documents that exist in this persistence instance.
     * <p/>
     * CS section 1.4.1.1 describes the algorithm of this method.
     *
     * @return set of WordSourceId of documents that exist in this persistence
     *
     * @throws IndexPersistenceException when the retrieval fails
     */
    public Set getIndexedDocuments() throws IndexPersistenceException {
        //arg checking and everything else is in common method
        return CommonDatabaseOperations.getIndexedDocuments(this);
    }

    /**
     * This method writes the word data from the given set to the given writer using the given delimiter.
     *
     * @param wordSet  the set of words to be written as set of {@link java.util.Map.Entry} having a {@link
     *                 CollationKey}as key and a {@link List} of {@link Integer}s as values
     * @param writer   the writer to write to
     * @param posDelim the delimiter to be used to separate entries in stream
     *
     * @throws IOException in case some I/O operation fails
     */
    private void writeWordDataToWriter(final Set wordSet, final Writer writer, final String posDelim)
        throws IOException {
        for (Iterator iterator = wordSet.iterator(); iterator.hasNext();) {
            final Map.Entry entry = (Map.Entry) iterator.next();

            // write the word
            final String word = ((CollationKey) entry.getKey()).getSourceString();

            // write out all positions
            final List indices = (List) entry.getValue();
            writer.write(word);
            writer.write(posDelim);
            for (Iterator iterator1 = indices.iterator(); iterator1.hasNext();) {
                final Integer index = (Integer) iterator1.next();
                writer.write(index.toString());
                writer.write(posDelim);
            }
            //end the record of the current word by writing second delimiter
            writer.write(posDelim);
        }
        writer.flush();
        writer.close();
    }

    /**
     * This method reads the serialized word data (as written by {@link #writeWordDataToWriter(java.util.Set,
     * java.io.Writer, String)}) back into a map containing keys of type {@link CollationKey} and values that are {@link
     * List}s of {@link Integer}s.
     *
     * @param characterStream the stream to read from
     * @param delim           the delimiter to be used to determine entry boundaries
     * @param collator        the collator to be used to convert Strings to {@link java.text.CollationKey}s
     *
     * @return the map containing the data parsed from the given stream
     *
     * @throws IOException               in case some I/O operation fails
     * @throws IndexPersistenceException in case unexpected data is encountered in the stream
     */
    private Map readWordDataFromReader(final Reader characterStream, final String delim, final Collator collator) throws
        IOException, IndexPersistenceException {
        final Map words = new HashMap();
        final BufferedReader bufferedReader = new BufferedReader(characterStream);
        int c = -1;
        // the following code is a simple state machine that finds the occurrences of
        // delimiters and puts the data into the words map

        final char[] delimChars = delim.toCharArray();
        // the string buffer collects all characters encountered since last delimiter occurrence
        StringBuffer item = new StringBuffer();
        // the index of how many of the last characters matched the delimiter sequence,
        // when this gets as large as delimChars.length, a delimiter has been found
        int delimIdx = 0;
        // the current word, null until the word end delimiter has been found,
        // if not null indicates that values read are indices for word
        String word = null;

        // the indices for the current word
        List indices = new ArrayList();

        // state flag signaling that one delimiter has been fully read, if the delimiter is
        // then immediately followed by a second delimiter, the word entry is over
        // and word plus indices are put to map
        boolean haveOneDelim = false;
        while ((c = bufferedReader.read()) != -1) {
            item.append((char) c);

            if (c == delimChars[delimIdx]) {
                delimIdx++; //match next delim char
            } else {
                delimIdx = 0; //does not match delim, so reset
                haveOneDelim = false;
            }

            if (delimIdx == delimChars.length) {
                // delim has been matched
                final String entry = item.toString().substring(0, item.length() - delimChars.length);
                if (word == null) {
                    word = entry;
                } else if (haveOneDelim) {
                    //second delim matched, so word entry is full
                    final CollationKey key = collator.getCollationKey(word);
                    words.put(key, indices);
                    indices = new ArrayList();
                    word = null;
                } else {
                    try {
                        indices.add(Integer.valueOf(entry.trim()));
                    } catch (NumberFormatException e) {
                        throw new IndexPersistenceException(
                            "Unable to parse a index string [" + entry + "] into a valid int.", e);
                    }
                }
                item = new StringBuffer();
                delimIdx = 0;
                haveOneDelim = true;
            }
        }
        if (item.length() > 0) {
            throw new IndexPersistenceException("Illegal end of stream detected during query");
        }
        return words;
    }
}
