/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.DBConnectionFactory;
import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.persistence.impl.Utils;
import com.topcoder.document.index.wordsource.WordSourceId;

import java.sql.Connection;
import java.text.CollationKey;
import java.text.Collator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;


/**
 * This is an implementation of the AbstractDBIndexPersistence that deals with a simple and straightforward, normalized
 * data based schema. This is provided for a user who is not as concerned about speed of the execution but is more
 * concerned about simple data mining capabilities for 3rd party API. Since all the data is in plain view (nothing
 * except for a serialized BLOB java object DOCUMENT.SOURCE_IDENTITY) this can be easily utilized.
 * <p/>
 * <b>Thread safety:</b>This class is no designed to be thread-safe.
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
public class SimpleAccessDBIndexPersistence extends AbstractDBIndexPersistence {

    /**
     * This constructor will populate the connectionFactory and connectionName information from configuration.
     *
     * @param namespace The namespace for connection-related properties
     *
     * @throws IllegalArgumentException if namespace param is <tt>null</tt> or an empty (trim'd) String
     * @throws PersistenceConfigurationException
     *                                  if there are configuration issues encountered
     * @see AbstractDBIndexPersistence#AbstractDBIndexPersistence(String)
     */
    public SimpleAccessDBIndexPersistence(final String namespace) throws PersistenceConfigurationException {
        // arg checking is done in super constructor
        super(namespace);
    }

    /**
     * This constructor will populate the connectionFactory and connectionName information from input parameters.
     *
     * @param connectionFactory DBConnectionFactory instance used to generate connections
     * @param connectionName    The name of the connection used for persistence operations
     *
     * @throws IllegalArgumentException if connectionFactory is <tt>null</tt>
     */
    public SimpleAccessDBIndexPersistence(final DBConnectionFactory connectionFactory, final String connectionName) {
        // arg checking is done in super constructor
        super(connectionFactory, connectionName);
    }

    /**
     * This method adds the given DocumentIndex to the persistence.
     * <p/>
     * CS section 1.4.1.3 describes the algorithm of this method.
     *
     * @param documentIndex WordSource representing the document to be persisted
     *
     * @throws IllegalArgumentException  when documentIndex is <tt>null</tt>
     * @throws IndexPersistenceException when fails to add DocumentIndex to the persistence or given index is already
     *                                   persisted
     */
    public void addDocumentIndex(final DocumentIndex documentIndex) throws IndexPersistenceException {
        if (documentIndex == null) {
            throw new IllegalArgumentException("The parameter named [documentIndex] was null.");
        }

        final Connection connection = getConnection();
        beginTransaction(connection);

        int idCounter = 1;
        final String documentId =
            CommonDatabaseOperations.insertDocumentHeaderData(this, documentIndex.getWordSourceId(), connection);

        // the validity of these values (CS 1.4.1.3) is guaranteed by the final class
        // DocumentIndex and its constructor logic
        final Map words = documentIndex.getWords();
        final Set wordSet = words.entrySet();
        for (Iterator iterator = wordSet.iterator(); iterator.hasNext();) {
            final Map.Entry entry = (Map.Entry) iterator.next();

            // insert the WORD record
            final Integer wordId = new Integer(idCounter++);
            DatabaseUtils.doDMLQuery(connection, this,
                "INSERT INTO WORD (WORD_ID,DOCUMENT_ID,WORD_ENTRY) VALUES (?,?,?)",
                new Object[]{
                    wordId,
                    documentId,
                    ((CollationKey) entry.getKey()).getSourceString()
                });

            // insert the WORD_POSITION records
            final List indices = (List) entry.getValue();
            for (Iterator iterator1 = indices.iterator(); iterator1.hasNext();) {
                final Integer index = (Integer) iterator1.next();

                DatabaseUtils.doDMLQuery(connection, this,
                    "INSERT INTO WORD_POSITION (DOCUMENT_ID,WORD_ID,WORD_POSITION) VALUES (?,?,?)",
                    new Object[]{
                        documentId,
                        wordId,
                        index
                    });
            }
        }

        commitTransaction(connection);
    }

    /**
     * This method does retrieve a DocumentIndex with the specified WordSourceId from the persistence. It does return
     * <tt>null</tt>, if DocumentIndex with the given WordSourceId is not found in the persistence.
     * <p/>
     * CS section 1.4.1.1 describes the algorithm of this method.
     *
     * @param wordSourceId WordSourceId of document to retrieve
     *
     * @return retrieved DocumentIndex, or <tt>null</tt> if document index with given WordSourceId is not found in the
     *         persistence
     *
     * @throws IllegalArgumentException  when wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to retrieve document index with the specified WordSourceId
     */
    public DocumentIndex getDocumentIndex(final WordSourceId wordSourceId)
        throws IndexPersistenceException {
        if (wordSourceId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }

        final String documentId = Utils.createIdString(wordSourceId);

        final Object[] queryArgsDocumentId = new Object[]{documentId};

        final Connection connection = getConnection();

        final Integer count = (Integer) DatabaseUtils.doSingleValueQuery(connection, this,
            "SELECT COUNT(*) FROM DOCUMENT WHERE DOCUMENT_ID=?", queryArgsDocumentId,
            DatabaseUtils.INTEGER_TYPE);
        if (count.intValue() == 0) {
            DatabaseUtils.closeSilently(connection);
            return null;
        }

        final Locale locale = wordSourceId.getSourceLocale();
        final Collator collator = Collator.getInstance(locale);

        final Map words = new HashMap();
        // fetch all WORDs
        final List result3 = DatabaseUtils.doQuery(connection, this,
            "SELECT WORD_ID,WORD_ENTRY FROM WORD WHERE DOCUMENT_ID=?",
            queryArgsDocumentId,
            new DatabaseUtils.DataType[]{DatabaseUtils.INTEGER_TYPE, DatabaseUtils.STRING_TYPE}
        );

        for (Iterator iterator = result3.iterator(); iterator.hasNext();) {
            final Object[] wordRecord = (Object[]) iterator.next();
            final Integer wordId = (Integer) wordRecord[0];
            final CollationKey key = collator.getCollationKey((String) wordRecord[1]);

            //fetch all indices
            final List result4 = DatabaseUtils.doSingleColumnQuery(connection, this,
                "SELECT WORD_POSITION FROM WORD_POSITION WHERE DOCUMENT_ID=? AND WORD_ID=?",
                new Object[]{documentId, wordId},
                DatabaseUtils.INTEGER_TYPE
            );
            words.put(key, result4);
        }
        DatabaseUtils.closeSilently(connection);
        return new DocumentIndex(wordSourceId, words);
    }

    /**
     * This method does remove a document index with given WordSourceId from the persistence. When the document index is
     * not found, its use count is greater that zero, or function fail to remove it exception is thrown.
     * <p/>
     * CS section 1.4.1.2 describes the algorithm of this method.
     *
     * @param wordSourceId WordSourceId of  document index to remove
     *
     * @throws IllegalArgumentException  when WordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when document index is not found in the persistence, its use count is not zero,
     *                                   or error happens when trying to remove it
     */
    public void removeDocumentIndex(final WordSourceId wordSourceId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        if (wordSourceId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }
        final String documentId = Utils.createIdString(wordSourceId);

        final Connection connection = getConnection();
        final Object[] documentIdQueryArg = new Object[]{documentId};
        final int count = getDocumentUseCount(wordSourceId);
        if (count != 0) {
            throw new IndexPersistenceException("The index with the given id [" + wordSourceId
                + "] cannot be deleted as its use count is not 0, but is " + count + ".");
        }

        beginTransaction(connection);

        DatabaseUtils.doDMLQuery(connection, this, "DELETE FROM WORD_POSITION WHERE DOCUMENT_ID=?",
            documentIdQueryArg);
        DatabaseUtils.doDMLQuery(connection, this, "DELETE FROM WORD WHERE DOCUMENT_ID=?", documentIdQueryArg);
        DatabaseUtils.doDMLQuery(connection, this, "DELETE FROM DELIMITER WHERE DOCUMENT_ID=?",
            documentIdQueryArg);
        DatabaseUtils.doDMLQuery(connection, this, "DELETE FROM DOCUMENT WHERE DOCUMENT_ID=?",
            documentIdQueryArg);
        // the DOCUMENT_COLLECTION_DOCUMENT_XREF table remains untouched, as the indexer component assures no
        // matching entry to exist in that table when use count of document is 0

        commitTransaction(connection);
    }

    /**
     * This method does store the given document collection index in the persistence.
     * <p/>
     * CS section 1.4.3.1 describes the algorithm of this method.
     *
     * @param collectionIndex CollectionIndex to store
     *
     * @throws IllegalArgumentException  if collectionIndex is <tt>null</tt>
     * @throws IndexPersistenceException if fails to store document collection index or given index is already
     *                                   persisted
     */
    public void addCollectionIndex(final CollectionIndex collectionIndex)
        throws IndexPersistenceException {
        //arg checking and everything else is in common method
        CommonDatabaseOperations.addCollectionIndex(this, collectionIndex);
    }

    /**
     * This method retrieves a document collection index with specified identifier. It does return <tt>null</tt> when
     * collection with specified identifier is not found.
     * <p/>
     * CS section 1.4.3.2 describes the algorithm of this method.
     *
     * @param collectionId identifier of document collection index to retrieve
     *
     * @return CollectionIndex with specified identifier or <tt>null</tt> if the collection index with the given
     *         identifier was not found in persistence
     *
     * @throws IllegalArgumentException  when collectionId is <tt>null</tt> or an empty (trim'd) string
     * @throws IndexPersistenceException when collection index can not be retrieved
     */
    public CollectionIndex getCollectionIndex(final String collectionId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        return CommonDatabaseOperations.getCollectionIndex(this, collectionId);
    }

    /**
     * This method does remove the document collection index with specified identifier from the persistence.
     * <p/>
     * CS section 1.4.3.4 describes the algorithm of this method.
     *
     * @param collectionId identifier of document collection index to remove
     *
     * @throws IllegalArgumentException  when collectionId is <tt>null</tt> or empty (trim'd) string
     * @throws IndexPersistenceException when fails to remove CollectionIndex with given identifier from database, or
     *                                   when collection with that specific identifier did not exist in database
     */
    public void removeCollectionIndex(final String collectionId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        CommonDatabaseOperations.removeCollectionIndex(this, collectionId);
    }

    /**
     * This method does update specified CollectionIndex in the persistence.
     * <p/>
     * CS section 1.4.3.3 describes the algorithm of this method.
     *
     * @param collectionIndex CollectionIndex to update
     *
     * @throws IllegalArgumentException  if collectionIndex parameter is <tt>null</tt>
     * @throws IndexPersistenceException if any error happens when updating the collection index in the persistence,
     *                                   this includes the case, when the specified collectionIndex is not found in the
     *                                   persistence
     */
    public void updateCollectionIndex(final CollectionIndex collectionIndex) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        CommonDatabaseOperations.updateCollectionIndex(this, collectionIndex);
    }

    /**
     * This method increases the use count value for document index with specified WordSourceId by 1.
     *
     * @param wordSourceId WordSourceId of document index of which to update use count
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to increase document index use count in the persistence or the
     *                                   document with the given wordSourceId does not exist in persistence
     */
    public void increaseDocumentUseCount(final WordSourceId wordSourceId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        CommonDatabaseOperations.increaseDocumentUseCount(this, wordSourceId, 1);
    }

    /**
     * This method decreases the use count value for document index with specified WordSourceId by 1.
     *
     * @param wordSourceId WordSourceId of document index of which to update use count
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to increase document index use count in the persistence or the
     *                                   document with the given wordSourceId does not exist in persistence
     */
    public void decreaseDocumentUseCount(final WordSourceId wordSourceId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        CommonDatabaseOperations.increaseDocumentUseCount(this, wordSourceId, -1);
    }

    /**
     * This method does return the use count of the document index with the given id.
     *
     * @param wordSourceId WordSourceId of document index of which to retrieve use count
     *
     * @return the use count of the document with the given id
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to retrieve document index use count in the persistence or the
     *                                   document with the given id does not exist in the persistence
     */
    public int getDocumentUseCount(final WordSourceId wordSourceId) throws IndexPersistenceException {
        //arg checking and everything else is in common method
        return CommonDatabaseOperations.getDocumentUseCount(this, wordSourceId);
    }

    /**
     * This method does return the set of WordSourceId of documents that exist in this persistence instance.
     * <p/>
     * CS section 1.4.1.1 describes the algorithm of this method.
     *
     * @return set of WordSourceId of documents that exist in this persistence
     *
     * @throws IndexPersistenceException when the retrieval fails
     */
    public Set getIndexedDocuments() throws IndexPersistenceException {
        //arg checking and everything else is in common method
        return CommonDatabaseOperations.getIndexedDocuments(this);
    }

}
