/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.impl.db.BaseDocumentIndexerPersistenceTest;
import com.topcoder.document.index.wordsource.WordSourceId;

import java.text.CollationKey;
import java.text.Collator;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Set;


/**
 * This abstract test class contains the test cases for IndexPersistence implementations.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public abstract class AbstractTestForIndexPersistence extends BaseDocumentIndexerPersistenceTest {
    /**
     * This constant is used in test cases as source identity value.
     */
    private static final String SOURCE_IDENTITY = "blah";
    /**
     * This constant is used in test cases as source identity value.
     */
    private static final String SOURCE_IDENTITY2 = "blah2";
    /**
     * This constant is used in test cases as source identity class name.
     */
    private static final String CLASS_NAME = "blubb";
    /**
     * This constant is used in test cases as source identity delimiters.
     */
    private static final String[] DELIMITERS = new String[]{"a", "c", "n"};
    /**
     * This constant is used in test cases as word of the index.
     */
    private static final String WORD = "word";
    /**
     * This constant is used in test cases as collection id.
     */
    private static final String COLLECTION_ID = "COLLID";

    /**
     * This is a locale used in test cases, initialized in {@link #setUp()}.
     */
    private Locale locale;
    /**
     * This is a WordSourceId used in test cases, initialized in {@link #setUp()}.
     */
    private WordSourceId sourceId;

    /**
     * This is a words map used in test cases, initialized in {@link #setUp()}.
     */
    private HashMap words;
    /**
     * This is a collator used in test cases, initialized in {@link #setUp()}.
     */
    private Collator collator;

    /**
     * This is a document index instance used in test cases, initialized in {@link #setUp()}.
     */
    private DocumentIndex index;

    /**
     * This is a collection index instance used in test cases, initialized in {@link #setUp()}.
     */
    private CollectionIndex collectionIndex;
    /**
     * This is the instance to be tested, initialized in {@link #setUp()}.
     */
    private IndexPersistence persistence;

    /**
     * This method tests {@link IndexPersistence#addDocumentIndex(DocumentIndex)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddDocumentIndex() throws Exception {
        persistence.addDocumentIndex(index);
        assertDocumentIndicesEqual(index, persistence.getDocumentIndex(sourceId));
    }

    /**
     * This method tests {@link IndexPersistence#addDocumentIndex(DocumentIndex)} for correctness.
     * <p/>
     * <b>Special Scenario:</b> a somewhat larger index is persisted
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddLargeDocumentIndex() throws Exception {
        locale = Locale.getDefault();
        sourceId = new WordSourceId(SOURCE_IDENTITY, CLASS_NAME, DELIMITERS, locale);
        words = new HashMap();
        final Set sourceIds = new HashSet();
        sourceIds.add(sourceId);
        collator = Collator.getInstance(locale);
        final ArrayList indices = new ArrayList();
        indices.add(new Integer(3));
        indices.add(new Integer(7));
        for (int i = 0; i < 1000; i++) {
            final CollationKey key = collator.getCollationKey(WORD + i);
            words.put(key, indices);
        }
        index = new DocumentIndex(sourceId, words);

        persistence.addDocumentIndex(index);
        assertDocumentIndicesEqual(index, persistence.getDocumentIndex(sourceId));
    }

    /**
     * This method tests {@link IndexPersistence#addDocumentIndex(DocumentIndex)} for correctness.
     * <p/>
     * <b>Special Scenario:</b> no words in index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddDocumentIndexNoWords() throws Exception {
        words.clear();
        index = new DocumentIndex(sourceId, words);
        persistence.addDocumentIndex(index);
        assertDocumentIndicesEqual(index, persistence.getDocumentIndex(sourceId));
    }

    /**
     * This method tests {@link IndexPersistence#addDocumentIndex(DocumentIndex)} for correctness.
     * <p/>
     * <b>Special Scenario:</b> delimiters are special chars that might need to be escaped
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddDocumentIndexSpecialDelimiters() throws Exception {
        sourceId = new WordSourceId(SOURCE_IDENTITY, CLASS_NAME,
            new String[]{"\n", "\r", " ", "\t", "\0", "<", ">", "\"", "&"}, locale);
        index = new DocumentIndex(sourceId, words);
        persistence.addDocumentIndex(index);
        assertDocumentIndicesEqual(index, persistence.getDocumentIndex(sourceId));
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#addDocumentIndex(DocumentIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given index argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddDocumentIndexFailureNull() throws Exception {
        try {
            persistence.addDocumentIndex(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#addDocumentIndex(DocumentIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given index argument is  already persisted
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddDocumentIndexFailureAlreadyPersisted() throws Exception {
        persistence.addDocumentIndex(index);
        try {
            persistence.addDocumentIndex(index);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link IndexPersistence#getDocumentIndex(WordSourceId)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetDocumentIndex() throws Exception {
        persistence.addDocumentIndex(index);
        assertDocumentIndicesEqual(index, persistence.getDocumentIndex(sourceId));
    }

    /**
     * This method tests {@link IndexPersistence#getDocumentIndex(WordSourceId)} for correctness.
     * <p/>
     * <b>Special Scenario</b>: the given id refers to an unpersisted index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetDocumentIndexUnpersisted() throws Exception {
        assertNull(persistence.getDocumentIndex(sourceId));
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#getDocumentIndex(WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetDocumentIndexFailureNull() throws Exception {
        try {
            persistence.getDocumentIndex(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link IndexPersistence#removeDocumentIndex(WordSourceId)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveDocumentIndex() throws Exception {
        persistence.addDocumentIndex(index);
        assertDocumentIndicesEqual(index, persistence.getDocumentIndex(sourceId));
        persistence.removeDocumentIndex(sourceId);
        assertNull(persistence.getDocumentIndex(sourceId));
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#removeDocumentIndex(WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveDocumentIndexFailureNull() throws Exception {
        try {
            persistence.removeDocumentIndex(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#removeDocumentIndex(WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument refers to an unpersisted document index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveDocumentIndexFailureUnpersisted() throws Exception {
        try {
            persistence.removeDocumentIndex(sourceId);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#removeDocumentIndex(WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument refers to an document index whose use count is greater
     * than 0
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveDocumentIndexFailureStillInUse() throws Exception {
        persistence.addDocumentIndex(index);
        persistence.increaseDocumentUseCount(sourceId);
        try {
            persistence.removeDocumentIndex(sourceId);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link IndexPersistence#addCollectionIndex(CollectionIndex)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndex() throws Exception {
        persistence.addDocumentIndex(index);
        persistence.addCollectionIndex(collectionIndex);

        assertEquals("use count must not be modified by collection persisting operation"
            , 0, persistence.getDocumentUseCount(sourceId));

        final CollectionIndex retrieved = persistence.getCollectionIndex(COLLECTION_ID);
        final CollectionIndex original = collectionIndex;
        assertCollectionIndicesEqual(original, retrieved);
    }

    /**
     * This method tests {@link IndexPersistence#addCollectionIndex(CollectionIndex)} for correctness.
     * <p/>
     * <b>Special Scenario</b>: the given index contains multiple documents
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndexWithMultipleDocuments() throws Exception {
        final WordSourceId sourceId2 = new WordSourceId(SOURCE_IDENTITY2, CLASS_NAME, DELIMITERS, locale);
        final DocumentIndex index2 = new DocumentIndex(sourceId2, words);

        final Set documentSet = new HashSet();
        documentSet.add(sourceId);
        documentSet.add(sourceId2);

        final Map indexWords = new HashMap();
        for (Iterator iterator = words.keySet().iterator(); iterator.hasNext();) {
            final Object o = iterator.next();
            indexWords.put(o, documentSet);
        }

        collectionIndex = new CollectionIndex(null, indexWords, documentSet, COLLECTION_ID);

        persistence.addDocumentIndex(index);
        persistence.addDocumentIndex(index2);

        persistence.addCollectionIndex(collectionIndex);

        assertEquals("use count must not be modified by collection persisting operation"
            , 0, persistence.getDocumentUseCount(sourceId));
        assertEquals("use count must not be modified by collection persisting operation"
            , 0, persistence.getDocumentUseCount(sourceId2));

        final CollectionIndex retrieved = persistence.getCollectionIndex(COLLECTION_ID);
        final CollectionIndex original = collectionIndex;
        assertCollectionIndicesEqual(original, retrieved);
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#addCollectionIndex(CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionIndex is <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndexFailureNull() throws Exception {
        try {
            persistence.addCollectionIndex(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#addCollectionIndex(CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionIndex refers to an unpersisted document index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndexFailureUnpersisted() throws Exception {
        try {
            persistence.addCollectionIndex(collectionIndex);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#addCollectionIndex(CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionIndex is already persisted
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndexFailureAlreadPersisted() throws Exception {
        persistence.addDocumentIndex(index);
        persistence.addCollectionIndex(collectionIndex);
        try {
            persistence.addCollectionIndex(collectionIndex);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link IndexPersistence#getCollectionIndex(String)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetCollectionIndex() throws Exception {
        persistence.addDocumentIndex(index);
        persistence.addCollectionIndex(collectionIndex);
        assertCollectionIndicesEqual(collectionIndex, persistence.getCollectionIndex(COLLECTION_ID));
    }

    /**
     * This method tests {@link IndexPersistence#getCollectionIndex(String)} for correctness.
     * <p/>
     * <b>Special Scenario</b>: the given id refers to an unpersisted index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetCollectionIndexUnpersisted() throws Exception {
        assertNull(persistence.getCollectionIndex(COLLECTION_ID));
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#getCollectionIndex(String)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetCollectionIndexFailureNull() throws Exception {
        try {
            persistence.getCollectionIndex(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#getCollectionIndex(String)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionId argument is  an empty String
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetCollectionIndexFailureEmpty() throws Exception {
        try {
            persistence.getCollectionIndex("  ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link IndexPersistence#removeCollectionIndex(String)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveCollectionIndex() throws Exception {
        persistence.addDocumentIndex(index);
        persistence.addCollectionIndex(collectionIndex);
        assertCollectionIndicesEqual(collectionIndex, persistence.getCollectionIndex(COLLECTION_ID));
        persistence.removeCollectionIndex(COLLECTION_ID);
        assertNull(persistence.getCollectionIndex(COLLECTION_ID));
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#removeCollectionIndex(String)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionId argument is  an empty String
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveCollectionIndexFailureEmpty() throws Exception {
        try {
            persistence.removeCollectionIndex("  ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#removeCollectionIndex(String)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveCollectionIndexFailureNull() throws Exception {
        try {
            persistence.removeCollectionIndex(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#removeCollectionIndex(String)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionId argument refers to an unpersisted collection index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveCollectionIndexFailureUnpersisted() throws Exception {
        try {
            persistence.removeCollectionIndex(COLLECTION_ID);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link IndexPersistence#updateCollectionIndex(CollectionIndex)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testUpdateCollectionIndex() throws Exception {
        persistence.addDocumentIndex(index);
        persistence.addCollectionIndex(collectionIndex);
        assertCollectionIndicesEqual(collectionIndex, persistence.getCollectionIndex(COLLECTION_ID));

        final WordSourceId sourceId2 = new WordSourceId(SOURCE_IDENTITY2, CLASS_NAME, DELIMITERS, locale);
        final DocumentIndex index2 = new DocumentIndex(sourceId2, words);

        final Set documentSet = new HashSet();
        documentSet.add(sourceId);
        documentSet.add(sourceId2);

        final Map indexWords = new HashMap();
        for (Iterator iterator = words.keySet().iterator(); iterator.hasNext();) {
            final Object o = iterator.next();
            indexWords.put(o, documentSet);
        }

        collectionIndex = new CollectionIndex(null, indexWords, documentSet, COLLECTION_ID);

        persistence.addDocumentIndex(index2);

        persistence.updateCollectionIndex(collectionIndex);

        assertEquals("use count must not be modified by collection persisting operation"
            , 0, persistence.getDocumentUseCount(sourceId));
        assertEquals("use count must not be modified by collection persisting operation"
            , 0, persistence.getDocumentUseCount(sourceId2));

        assertCollectionIndicesEqual(collectionIndex, persistence.getCollectionIndex(COLLECTION_ID));
    }

    /**
     * This method tests the correct failure behavior of
     * <p/>
     * {@link IndexPersistence#updateCollectionIndex(CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given collection argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testUpdateCollectionIndexFailureNull() throws Exception {
        try {
            persistence.updateCollectionIndex(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of
     * <p/>
     * {@link IndexPersistence#updateCollectionIndex(CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given collection argument refers to an unpersisted collection index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testUpdateCollectionIndexFailureUnpersisted() throws Exception {
        try {
            persistence.updateCollectionIndex(collectionIndex);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link IndexPersistence#increaseDocumentUseCount(WordSourceId)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testIncreaseDocumentUseCount() throws Exception {
        persistence.addDocumentIndex(index);
        persistence.increaseDocumentUseCount(sourceId);
        assertEquals("use count increase failed", 1, persistence.getDocumentUseCount(sourceId));
        persistence.increaseDocumentUseCount(sourceId);
        assertEquals("use count increase failed", 2, persistence.getDocumentUseCount(sourceId));
    }

    /**
     * This method tests the correct failure behavior of
     * <p/>
     * {@link IndexPersistence#increaseDocumentUseCount(WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testIncreaseDocumentUseCountFailureNull() throws Exception {
        try {
            persistence.increaseDocumentUseCount(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of
     * <p/>
     * {@link IndexPersistence#increaseDocumentUseCount(WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument refers to an unpersisted document index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testIncreaseDocumentUseCountFailureUnpersisted() throws Exception {
        try {
            persistence.increaseDocumentUseCount(sourceId);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link IndexPersistence#decreaseDocumentUseCount(WordSourceId)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDecreaseDocumentUseCount() throws Exception {
        persistence.addDocumentIndex(index);
        persistence.decreaseDocumentUseCount(sourceId);
        assertEquals("use count decrease failed", -1, persistence.getDocumentUseCount(sourceId));
        persistence.decreaseDocumentUseCount(sourceId);
        assertEquals("use count decrease failed", -2, persistence.getDocumentUseCount(sourceId));
    }

    /**
     * This method tests the correct failure behavior of
     * <p/>
     * {@link IndexPersistence#decreaseDocumentUseCount(WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDecreaseDocumentUseCountFailureNull() throws Exception {
        try {
            persistence.decreaseDocumentUseCount(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of
     * <p/>
     * {@link IndexPersistence#decreaseDocumentUseCount(WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument refers to an unpersisted document index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDecreaseDocumentUseCountFailureUnpersisted() throws Exception {
        try {
            persistence.decreaseDocumentUseCount(sourceId);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link IndexPersistence#getDocumentUseCount(WordSourceId)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetDocumentUseCount() throws Exception {
        persistence.addDocumentIndex(index);
        persistence.increaseDocumentUseCount(sourceId);
        assertEquals("use count get failed", 1, persistence.getDocumentUseCount(sourceId));
        persistence.increaseDocumentUseCount(sourceId);
        assertEquals("use count get failed", 2, persistence.getDocumentUseCount(sourceId));
        persistence.decreaseDocumentUseCount(sourceId);
        assertEquals("use count get failed", 1, persistence.getDocumentUseCount(sourceId));
        persistence.decreaseDocumentUseCount(sourceId);
        assertEquals("use count get failed", 0, persistence.getDocumentUseCount(sourceId));
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#getDocumentUseCount(WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetDocumentUseCountFailureNull() throws Exception {
        try {
            persistence.getDocumentUseCount(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link IndexPersistence#getDocumentUseCount(WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument refers to an unpersisted document index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetDocumentUseCountFailureUnpersisted() throws Exception {
        try {
            persistence.getDocumentUseCount(sourceId);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link IndexPersistence#getIndexedDocuments()} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetIndexedDocuments() throws Exception {
        Set set = persistence.getIndexedDocuments();
        assertTrue("indexed documents should be empty", set.isEmpty());

        persistence.addDocumentIndex(index);
        set = persistence.getIndexedDocuments();
        assertEquals("indexed documents size does not match", 1, set.size());
        assertTrue("a document is missing", set.contains(sourceId));

        final WordSourceId sourceId2 = new WordSourceId(SOURCE_IDENTITY2, CLASS_NAME, DELIMITERS, locale);
        final DocumentIndex index2 = new DocumentIndex(sourceId2, words);
        persistence.addDocumentIndex(index2);
        set = persistence.getIndexedDocuments();
        assertEquals("indexed documents size does not match", 2, set.size());
        assertTrue("a document is missing", set.contains(sourceId));
        assertTrue("a document is missing", set.contains(sourceId2));
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        persistence = createPersistenceInstance();
        locale = Locale.getDefault();
        sourceId = new WordSourceId(SOURCE_IDENTITY, CLASS_NAME, DELIMITERS, locale);
        words = new HashMap();
        final Set sourceIds = new HashSet();
        sourceIds.add(sourceId);
        final HashMap collectionWords = new HashMap();
        collator = Collator.getInstance(locale);
        CollationKey key = collator.getCollationKey(WORD);
        final ArrayList indices = new ArrayList();
        indices.add(new Integer(3));
        indices.add(new Integer(7));
        words.put(key, indices);
        collectionWords.put(key, sourceIds);
        key = collator.getCollationKey("Word2");
        words.put(key, indices);
        collectionWords.put(key, sourceIds);
        key = collator.getCollationKey("wo<>\'\"rd3");
        indices.add(new Integer(9));
        words.put(key, indices);
        collectionWords.put(key, sourceIds);
        index = new DocumentIndex(sourceId, words);
        final HashSet documents = new HashSet();
        documents.add(sourceId);
        collectionIndex = new CollectionIndex(null, collectionWords, documents, COLLECTION_ID);
    }

    /**
     * This method creates the instance to be tested in the test cases. Subclasses override this method to return the
     * instance to be tested here.
     *
     * @return the instance to be tested
     *
     * @throws Exception in case some unexpected exception occurs
     */
    protected abstract IndexPersistence createPersistenceInstance() throws Exception;
}
