/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.IndexManager;
import com.topcoder.document.index.persistence.IndexPersistence;
import com.topcoder.document.index.persistence.impl.db.BaseDocumentIndexerPersistenceTest;
import com.topcoder.document.index.persistence.impl.db.FastAccessDBIndexPersistence;
import com.topcoder.document.index.persistence.impl.db.SimpleAccessDBIndexPersistence;
import com.topcoder.document.index.persistence.impl.xml.XmlIndexPersistence;
import com.topcoder.document.index.wordsource.WordSourceId;

import java.text.CollationKey;
import java.text.Collator;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Locale;
import java.util.Set;


/**
 * This class contains the usage demo for the Document Indexer persistence component.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class Demo extends BaseDocumentIndexerPersistenceTest {
    /**
     * This constant is used in test cases as collection id.
     */
    private static final String COLLECTION_ID = "COLLID";

    /**
     * This is a WordSourceId used in test cases, initialized in {@link #setUp()}.
     */
    private WordSourceId sourceId;

    /**
     * This is a document index instance used in test cases, initialized in {@link #setUp()}.
     */
    private DocumentIndex index;

    /**
     * This is a collection index instance used in test cases, initialized in {@link #setUp()}.
     */
    private CollectionIndex collectionIndex;

    /**
     * This method shows how the different persistence implementations can be plugged into the persistence manager and
     * will be used by that manager.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testManagerBasedPersistence() throws Exception {
        // Create a XML based persistence
        final IndexPersistence xmlPersistence = new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
        // pass it to the manager
        IndexManager manager = new IndexManager(xmlPersistence);
        // // Create a fast db based persistence
        final IndexPersistence fastDBPersistence = new FastAccessDBIndexPersistence(FAST_DB_CONFIGURATION_NAMESPACE);
        // // pass it to the manager
        manager = new IndexManager(fastDBPersistence);
        // // Create a simple db based persistence
        final IndexPersistence simpleDBPersistence = new SimpleAccessDBIndexPersistence(
            SIMPLE_DB_CONFIGURATION_NAMESPACE);
        // // pass it to the manager
        manager = new IndexManager(simpleDBPersistence);

        // at this point the manager can be used to do indexing operations and
        // that manager will delegate persistence to the persistence instance.
        //
        // Example:
        // manager.createDocumentIndex(new TextFileWordSource(....))
        // For detailed examples on the Document Indexer component see the Document Indexer CS
    }

    /**
     * This method shows how the different persistence implementations can be use directly to store and retrieve
     * documents and collections.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDirectPersistence() throws Exception {
        // Create a persistence (this is the XML version, but we can use any persistence implementation)
        final IndexPersistence persistence = new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);

        // The data used in these method is created in setUp manually, normally the data would be created by
        // the DocumentIndexer implementation, but for the test case we use manual creation

        // initially persist a document
        persistence.addDocumentIndex(index);

        // now we can retrieve the document
        final DocumentIndex retrieved = persistence.getDocumentIndex(sourceId);

        // we can now persist the collection containing the document
        persistence.addCollectionIndex(collectionIndex);

        // we can increase, retrieve and decrease the use count
        persistence.increaseDocumentUseCount(sourceId);
        final int useCount = persistence.getDocumentUseCount(sourceId);
        persistence.decreaseDocumentUseCount(sourceId);

        // we can delete the collection (which does not delete the contained documents)
        persistence.removeCollectionIndex(COLLECTION_ID);

        // we can retrieve the ids of all documents present in the persistence
        final Set set = persistence.getIndexedDocuments();

        // and finally we can delete the document index from persistence
        persistence.removeDocumentIndex(sourceId);
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        final Locale locale = Locale.getDefault();
        sourceId = new WordSourceId("blah", "blubb", new String[]{"a", "c", "n"}, locale);
        final HashMap words = new HashMap();
        final Set sourceIds = new HashSet();
        sourceIds.add(sourceId);
        final HashMap collectionWords = new HashMap();
        final Collator collator = Collator.getInstance(locale);
        CollationKey key = collator.getCollationKey("word");
        final ArrayList indices = new ArrayList();
        indices.add(new Integer(3));
        indices.add(new Integer(7));
        words.put(key, indices);
        collectionWords.put(key, sourceIds);
        key = collator.getCollationKey("Word2");
        words.put(key, indices);
        collectionWords.put(key, sourceIds);
        key = collator.getCollationKey("word3");
        indices.add(new Integer(9));
        words.put(key, indices);
        collectionWords.put(key, sourceIds);
        index = new DocumentIndex(sourceId, words);
        final HashSet documents = new HashSet();
        documents.add(sourceId);
        collectionIndex = new CollectionIndex(null, collectionWords, documents, COLLECTION_ID);
    }
}
