/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.DBConnectionFactory;
import com.topcoder.db.connectionfactory.DBConnectionFactoryImpl;
import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.wordsource.WordSourceId;

import java.sql.Connection;
import java.util.Set;


/**
 * This class contains the test cases for the methods defined in {@link AbstractDBIndexPersistence}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class AbstractDBIndexPersistenceTest extends BaseDocumentIndexerPersistenceTest {
    /**
     * This is the instance to be tested, created in {@link #setUp()}.
     */
    private MockDBIndexPersistence persistence;

    /**
     * This method tests {@link AbstractDBIndexPersistence#AbstractDBIndexPersistence(String)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtor() throws Exception {
        new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
        loadSimpleConfiguration("simple_noConnectionName.xml");
        new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
        loadSimpleConfiguration("simple_emptyConnectionName.xml");
        new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
        loadSimpleConfiguration("simple_customConnectionClass.xml");
        new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence#AbstractDBIndexPersistence(
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the given namespace argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNull() throws Exception {
        try {
            new MockDBIndexPersistence(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence#AbstractDBIndexPersistence(
     * String)}.
     * <p/>
     * <b>Failure reason</b>: no configuration has been loaded
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNoConfig() throws Exception {
        clearConfig();
        try {
            new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence#AbstractDBIndexPersistence(
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for connectionFactoryNamespace is an empty String
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailEmptyConnectionFactoryNamespace() throws Exception {
        loadSimpleConfiguration("simple_emptyConnectionFactoryNamespace.xml");
        try {
            new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence#AbstractDBIndexPersistence(
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for connectionFactoryNamespace does not exist
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNoConnectionFactoryNamespace() throws Exception {
        loadSimpleConfiguration("simple_noConnectionFactoryNamespace.xml");
        try {
            new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence#AbstractDBIndexPersistence(
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for connectionFactoryNamespace does not refer to a valid CM
     * namespace
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailInvalidConnectionFactoryNamespace() throws Exception {
        loadSimpleConfiguration("simple_invalidConnectionFactoryNamespace.xml");
        try {
            new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence#AbstractDBIndexPersistence(
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for connectionFactoryClass refers to an unknown class
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailUnknownConnectionFactoryClass() throws Exception {
        loadSimpleConfiguration("simple_unknownConnectionFactoryClass.xml");
        try {
            new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence#AbstractDBIndexPersistence(
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for connectionFactoryClass refers to a class that has no
     * matching constructor
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNoMatchingCtorConnectionFactoryClass() throws Exception {
        loadSimpleConfiguration("simple_noMatchingCtorConnectionFactoryClass.xml");
        try {
            new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence#AbstractDBIndexPersistence(
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for connectionFactoryClass refers to a class that is of wrong
     * type
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailWrongTypeConnectionFactoryClass() throws Exception {
        loadSimpleConfiguration("simple_wrongTypeConnectionFactoryClass.xml");
        try {
            new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link AbstractDBIndexPersistence#AbstractDBIndexPersistence(DBConnectionFactory, String)} for
     * correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testTwoArgCtor() throws Exception {
        new MockDBIndexPersistence(new DBConnectionFactoryImpl(DEFAULT_DB_CONNECTION_FACTORY_NAMESPACE),
            "simple");
        new MockDBIndexPersistence(new DBConnectionFactoryImpl(DEFAULT_DB_CONNECTION_FACTORY_NAMESPACE), "  ");
        new MockDBIndexPersistence(new DBConnectionFactoryImpl(DEFAULT_DB_CONNECTION_FACTORY_NAMESPACE), null);
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence
     * #AbstractDBIndexPersistence(DBConnectionFactory, String)}.
     * <p/>
     * <b>Failure reason</b>: the given connectionFactory is <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testTwoArgCtorFailNull() throws Exception {
        try {
            new MockDBIndexPersistence(null, SIMPLE_DB_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link AbstractDBIndexPersistence#rollbackTransaction(java.sql.Connection)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRollbackTransaction() throws Exception {
        Connection connection = persistence.getConnection();

        try {
            DatabaseUtils.doDMLQuery(connection, persistence,
                "INSERT INTO DOCUMENT_COLLECTION (COLLECTION_ID) VALUES (?)",
                new Object[]{"id"});

            persistence.rollbackTransaction(connection);
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
        connection = persistence.getConnection();
        try {
            final Object result = DatabaseUtils.doSingleValueQuery(connection, persistence,
                "SELECT COUNT(*) FROM DOCUMENT_COLLECTION",
                new Object[0], DatabaseUtils.INTEGER_TYPE);
            assertEquals(0, ((Integer) result).intValue());
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence
     * #rollbackTransaction(java.sql.Connection)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> argument given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRollbackTransactionFailureNull() throws Exception {
        try {
            persistence.rollbackTransaction(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link AbstractDBIndexPersistence#commitTransaction(java.sql.Connection)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCommit() throws Exception {
        Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doDMLQuery(connection, persistence,
                "INSERT INTO DOCUMENT_COLLECTION (COLLECTION_ID) VALUES (?)",
                new Object[]{"id"});

            persistence.commitTransaction(connection);
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
        connection = persistence.getConnection();
        try {
            final Object result = DatabaseUtils.doSingleValueQuery(connection, persistence,
                "SELECT COUNT(*) FROM DOCUMENT_COLLECTION",
                new Object[0], DatabaseUtils.INTEGER_TYPE);
            assertEquals(1, ((Integer) result).intValue());
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence#
     * commitTransaction(java.sql.Connection)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> argument given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCommitFailureNull() throws Exception {
        try {
            persistence.commitTransaction(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence
     * #commitTransaction(java.sql.Connection)}.
     * <p/>
     * <b>Failure reason</b>: closed connection given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCommitFailureClosedConnection() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            connection.close();
            try {
                persistence.commitTransaction(connection);
                fail("should throw");
            } catch (IndexPersistenceException expected) {
                //expected
            }
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests {@link AbstractDBIndexPersistence#beginTransaction(java.sql.Connection)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testBeginTransaction() throws Exception {
        final Connection connection = persistence.getConnection();
        persistence.beginTransaction(connection);
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractDBIndexPersistence
     * #beginTransaction(Connection)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> argument given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testBeginTransactionFailureNull() throws Exception {
        try {
            persistence.beginTransaction(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link AbstractDBIndexPersistence#getConnection()} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testgetConnection() throws Exception {
        assertNotNull(persistence.getConnection());
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        persistence = new MockDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
    }

    /**
     * This class is a mock-up that does nothing but is used to create an instantiatable AbstractDBIndexPersistence for
     * testing.
     */
    private static final class MockDBIndexPersistence extends AbstractDBIndexPersistence {

        private MockDBIndexPersistence(final String namespace) throws PersistenceConfigurationException {
            super(namespace);
        }

        private MockDBIndexPersistence(final DBConnectionFactory connectionFactory, final String connectionName) {
            super(connectionFactory, connectionName);
        }

        public void addDocumentIndex(final DocumentIndex documentIndex) throws IndexPersistenceException {
            //this method is empty intentionally.
        }

        public DocumentIndex getDocumentIndex(final WordSourceId wordSourceId) throws IndexPersistenceException {
            //this method is empty intentionally.
            return null;
        }

        public void removeDocumentIndex(final WordSourceId wordSourceId) throws IndexPersistenceException {
            //this method is empty intentionally.
        }

        public void addCollectionIndex(final CollectionIndex collectionIndex) throws IndexPersistenceException {
            //this method is empty intentionally.
        }

        public CollectionIndex getCollectionIndex(final String collectionId) throws IndexPersistenceException {
            //this method is empty intentionally.
            return null;
        }

        public void removeCollectionIndex(final String collectionId) throws IndexPersistenceException {
            //this method is empty intentionally.
        }

        public void updateCollectionIndex(final CollectionIndex collectionIndex) throws IndexPersistenceException {
            //this method is empty intentionally.
        }

        public void increaseDocumentUseCount(final WordSourceId wordSourceId) throws IndexPersistenceException {
            //this method is empty intentionally.
        }

        public void decreaseDocumentUseCount(final WordSourceId wordSourceId) throws IndexPersistenceException {
            //this method is empty intentionally.
        }

        public int getDocumentUseCount(final WordSourceId wordSourceId) throws IndexPersistenceException {
            //this method is empty intentionally.
            return 0;
        }

        public Set getIndexedDocuments() throws IndexPersistenceException {
            //this method is empty intentionally.
            return null;
        }
    }
}
