/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.Utils;
import com.topcoder.document.index.wordsource.WordSourceId;

import java.sql.Connection;
import java.text.CollationKey;
import java.text.Collator;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Set;


/**
 * This test class contains the test cases for {@link CommonDatabaseOperations}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public abstract class CommonDatabaseOperationsTest extends BaseDocumentIndexerPersistenceTest {
    /**
     * This constant is used in test cases as source identity value.
     */
    private static final String SOURCE_IDENTITY = "blah";
    /**
     * This constant is used in test cases as source identity value.
     */
    private static final String SOURCE_IDENTITY2 = "blah2";
    /**
     * This constant is used in test cases as source identity class name.
     */
    private static final String CLASS_NAME = "blubb";
    /**
     * This constant is used in test cases as source identity delimiters.
     */
    private static final String[] DELIMITERS = new String[]{"a", "c", "n"};
    /**
     * This constant is used in test cases as word of the index.
     */
    private static final String WORD = "word";
    /**
     * This constant is used in test cases as word of the index.
     */
    private static final String WORD2 = "Word2";
    /**
     * This constant is used in test cases as word of the index.
     */
    private static final String WORD3 = "word3";
    /**
     * This constant is used in test cases as collection id.
     */
    private static final String COLLECTION_ID = "COLLID";

    /**
     * This is a locale used in test cases, initialized in {@link #setUp()}.
     */
    private Locale locale;
    /**
     * This is a WordSourceId used in test cases, initialized in {@link #setUp()}.
     */
    private WordSourceId sourceId;

    /**
     * This is a words map used in test cases, initialized in {@link #setUp()}.
     */
    private HashMap words;

    /**
     * This is a document index instance used in test cases, initialized in {@link #setUp()}.
     */
    private DocumentIndex index;

    /**
     * This is a collection index instance used in test cases, initialized in {@link #setUp()}.
     */
    private CollectionIndex collectionIndex;
    /**
     * This is a {@link AbstractDBIndexPersistence} instance used in tests, initialized in {@link #setUp()}.
     */
    private AbstractDBIndexPersistence persistence;
    /**
     * This is a database connection used in test cases, created in {@link #setUp()}, closed in {@link #tearDown()}.
     */
    private Connection connection;

    /**
     * This method tests {@link CommonDatabaseOperations#insertDocumentHeaderData(AbstractDBIndexPersistence,
     * com.topcoder.document.index.wordsource.WordSourceId, java.sql.Connection)} and {@link
     * CommonDatabaseOperations#getWordSourceId(AbstractDBIndexPersistence, java.sql.Connection, String)}for
     * correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testReadWriteWordSourceId() throws Exception {
        final String docId = CommonDatabaseOperations.insertDocumentHeaderData(persistence, sourceId, connection);
        final WordSourceId sourceId1 = CommonDatabaseOperations.getWordSourceId(persistence, connection, docId);
        connection.commit();
        assertEquals("source id must match", sourceId, sourceId1);
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#insertDocumentHeaderData(
     * AbstractDBIndexPersistence, com.topcoder.document.index.wordsource.WordSourceId, java.sql.Connection)}.
     * <p/>
     * <b>Failure reason</b>: the given persistence argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testInsertHeaderPersistenceNull() throws Exception {
        try {
            CommonDatabaseOperations.insertDocumentHeaderData(null, sourceId, connection);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            connection.commit();
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#insertDocumentHeaderData(
     * AbstractDBIndexPersistence, com.topcoder.document.index.wordsource.WordSourceId, java.sql.Connection)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testInsertHeaderWordSourceIdNull() throws Exception {
        try {
            CommonDatabaseOperations.insertDocumentHeaderData(persistence, null, connection);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#insertDocumentHeaderData(
     * AbstractDBIndexPersistence, com.topcoder.document.index.wordsource.WordSourceId, java.sql.Connection)}.
     * <p/>
     * <b>Failure reason</b>: the given connection argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testInsertHeaderConnectionNull() throws Exception {
        try {
            CommonDatabaseOperations.insertDocumentHeaderData(persistence, sourceId, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#insertDocumentHeaderData(
     * AbstractDBIndexPersistence, com.topcoder.document.index.wordsource.WordSourceId, java.sql.Connection)}.
     * <p/>
     * <b>Failure reason</b>: the given index argument is  already persisted
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddDocumentIndexFailureAlreadyPersisted() throws Exception {
        persistence.addDocumentIndex(index);
        try {
            CommonDatabaseOperations.insertDocumentHeaderData(persistence, sourceId, connection);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link CommonDatabaseOperations#getWordSourceId(AbstractDBIndexPersistence,
     * java.sql.Connection, String)} for correctness.
     * <p/>
     * <b>Special Scenario</b>: the given id refers to an unpersisted index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetWordSourceIdUnpersisted() throws Exception {
        assertNull(CommonDatabaseOperations.getWordSourceId(persistence, connection, Utils.createIdString(sourceId)));
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#getWordSourceId(
     * AbstractDBIndexPersistence, java.sql.Connection, String)}.
     * <p/>
     * <b>Failure reason</b>: the given persistence argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetWordSourceIdFailureNullPersistence() throws Exception {
        try {
            CommonDatabaseOperations.getWordSourceId(null, connection, Utils.createIdString(sourceId));
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#getWordSourceId(
     * AbstractDBIndexPersistence, java.sql.Connection, String)}.
     * <p/>
     * <b>Failure reason</b>: the given connection argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetWordSourceIdFailureNullConnection() throws Exception {
        try {
            CommonDatabaseOperations.getWordSourceId(persistence, null, Utils.createIdString(sourceId));
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#getWordSourceId(
     * AbstractDBIndexPersistence, java.sql.Connection, String)}.
     * <p/>
     * <b>Failure reason</b>: the given documentId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetWordSourceIdFailureNullId() throws Exception {
        try {
            CommonDatabaseOperations.getWordSourceId(persistence, connection, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#getWordSourceId(
     * AbstractDBIndexPersistence, java.sql.Connection, String)}.
     * <p/>
     * <b>Failure reason</b>: the given documentId argument is an empty String
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetWordSourceIdFailureEmptyId() throws Exception {
        try {
            CommonDatabaseOperations.getWordSourceId(persistence, connection, "  ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link CommonDatabaseOperations#addCollectionIndex(AbstractDBIndexPersistence,
     * com.topcoder.document.index.CollectionIndex)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndex() throws Exception {
        persistence.addDocumentIndex(index);
        CommonDatabaseOperations.addCollectionIndex(persistence, collectionIndex);

        assertEquals("use count must not be modified by collection persisting operation"
            , 0, CommonDatabaseOperations.getDocumentUseCount(persistence, sourceId));

        assertCollectionIndicesEqual(collectionIndex, CommonDatabaseOperations.getCollectionIndex(persistence,
            COLLECTION_ID));
    }

    /**
     * This method tests {@link CommonDatabaseOperations#addCollectionIndex(AbstractDBIndexPersistence,
     * com.topcoder.document.index.CollectionIndex)} for correctness.
     * <p/>
     * <b>Special Scenario</b>: the given index contains multiple documents
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndexWithMultipleDocuments() throws Exception {
        final WordSourceId sourceId2 = new WordSourceId(SOURCE_IDENTITY2, CLASS_NAME, DELIMITERS, locale);
        final DocumentIndex index2 = new DocumentIndex(sourceId2, words);

        final Set documentSet = new HashSet();
        documentSet.add(sourceId);
        documentSet.add(sourceId2);

        final Map indexWords = new HashMap();
        for (Iterator iterator = words.keySet().iterator(); iterator.hasNext();) {
            final Object o = iterator.next();
            indexWords.put(o, documentSet);
        }

        collectionIndex = new CollectionIndex(null, indexWords, documentSet, COLLECTION_ID);

        persistence.addDocumentIndex(index);
        persistence.addDocumentIndex(index2);

        CommonDatabaseOperations.addCollectionIndex(persistence, collectionIndex);

        assertEquals("use count must not be modified by collection persisting operation"
            , 0, CommonDatabaseOperations.getDocumentUseCount(persistence, sourceId));
        assertEquals("use count must not be modified by collection persisting operation"
            , 0, CommonDatabaseOperations.getDocumentUseCount(persistence, sourceId2));

        final CollectionIndex retrieved = CommonDatabaseOperations.getCollectionIndex(persistence, COLLECTION_ID);
        final CollectionIndex original = collectionIndex;
        assertCollectionIndicesEqual(original, retrieved);
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#addCollectionIndex(
     * AbstractDBIndexPersistence, com.topcoder.document.index.CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionIndex is <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndexFailureNullIndex() throws Exception {
        try {
            CommonDatabaseOperations.addCollectionIndex(persistence, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#addCollectionIndex(
     * AbstractDBIndexPersistence, com.topcoder.document.index.CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given persistence is <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndexFailureNullPersistence() throws Exception {
        try {
            CommonDatabaseOperations.addCollectionIndex(null, collectionIndex);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#addCollectionIndex(
     * AbstractDBIndexPersistence, com.topcoder.document.index.CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionIndex refers to an unpersisted document index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndexFailureUnpersisted() throws Exception {
        try {
            CommonDatabaseOperations.addCollectionIndex(persistence, collectionIndex);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#addCollectionIndex(
     * AbstractDBIndexPersistence, com.topcoder.document.index.CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionIndex is already persisted
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndexFailureAlreadPersisted() throws Exception {
        persistence.addDocumentIndex(index);
        CommonDatabaseOperations.addCollectionIndex(persistence, collectionIndex);
        try {
            CommonDatabaseOperations.addCollectionIndex(persistence, collectionIndex);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link CommonDatabaseOperations#getCollectionIndex(AbstractDBIndexPersistence, String)} for
     * correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetCollectionIndex() throws Exception {
        persistence.addDocumentIndex(index);
        CommonDatabaseOperations.addCollectionIndex(persistence, collectionIndex);
        assertCollectionIndicesEqual(collectionIndex, CommonDatabaseOperations.getCollectionIndex(persistence,
            COLLECTION_ID));
    }

    /**
     * This method tests {@link CommonDatabaseOperations#getCollectionIndex(AbstractDBIndexPersistence, String)} for
     * correctness.
     * <p/>
     * <b>Special Scenario</b>: the given id refers to an unpersisted index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetCollectionIndexUnpersisted() throws Exception {
        assertNull(CommonDatabaseOperations.getCollectionIndex(persistence, COLLECTION_ID));
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#getCollectionIndex(
     * AbstractDBIndexPersistence, String)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetCollectionIndexFailureNullId() throws Exception {
        try {
            CommonDatabaseOperations.getCollectionIndex(persistence, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#getCollectionIndex(
     * AbstractDBIndexPersistence, String)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionId argument is  an empty String
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetCollectionIndexFailureEmptyId() throws Exception {
        try {
            CommonDatabaseOperations.getCollectionIndex(persistence, "  ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#getCollectionIndex(
     * AbstractDBIndexPersistence, String)}.
     * <p/>
     * <b>Failure reason</b>: the given persistence argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetCollectionIndexFailureNullPersistence() throws Exception {
        try {
            CommonDatabaseOperations.getCollectionIndex(null, COLLECTION_ID);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link CommonDatabaseOperations#removeCollectionIndex(AbstractDBIndexPersistence, String)} for
     * correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveCollectionIndex() throws Exception {
        persistence.addDocumentIndex(index);
        CommonDatabaseOperations.addCollectionIndex(persistence, collectionIndex);
        assertCollectionIndicesEqual(collectionIndex, CommonDatabaseOperations.getCollectionIndex(persistence,
            COLLECTION_ID));
        CommonDatabaseOperations.removeCollectionIndex(persistence, COLLECTION_ID);
        assertNull(CommonDatabaseOperations.getCollectionIndex(persistence, COLLECTION_ID));
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#removeCollectionIndex(
     * AbstractDBIndexPersistence, String)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveCollectionIndexFailureNullCollectionId() throws Exception {
        try {
            CommonDatabaseOperations.removeCollectionIndex(persistence, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#removeCollectionIndex(
     * AbstractDBIndexPersistence, String)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionId argument is  an empty String
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveCollectionIndexFailureEmptyCollectionId() throws Exception {
        try {
            CommonDatabaseOperations.removeCollectionIndex(persistence, "  ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#removeCollectionIndex(
     * AbstractDBIndexPersistence, String)}.
     * <p/>
     * <b>Failure reason</b>: the given persistence argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveCollectionIndexFailureNullPersistence() throws Exception {
        try {
            CommonDatabaseOperations.removeCollectionIndex(null, COLLECTION_ID);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#removeCollectionIndex(
     * AbstractDBIndexPersistence, String)}.
     * <p/>
     * <b>Failure reason</b>: the given collectionId argument refers to an unpersisted collection index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testRemoveCollectionIndexFailureUnpersisted() throws Exception {
        try {
            CommonDatabaseOperations.removeCollectionIndex(persistence, COLLECTION_ID);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link CommonDatabaseOperations#updateCollectionIndex(AbstractDBIndexPersistence,
     * com.topcoder.document.index.CollectionIndex)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testUpdateCollectionIndex() throws Exception {
        persistence.addDocumentIndex(index);
        CommonDatabaseOperations.addCollectionIndex(persistence, collectionIndex);
        assertCollectionIndicesEqual(collectionIndex, CommonDatabaseOperations.getCollectionIndex(persistence,
            COLLECTION_ID));

        final WordSourceId sourceId2 = new WordSourceId(SOURCE_IDENTITY2, CLASS_NAME, DELIMITERS, locale);
        final DocumentIndex index2 = new DocumentIndex(sourceId2, words);

        final Set documentSet = new HashSet();
        documentSet.add(sourceId);
        documentSet.add(sourceId2);

        final Map indexWords = new HashMap();
        for (Iterator iterator = words.keySet().iterator(); iterator.hasNext();) {
            final Object o = iterator.next();
            indexWords.put(o, documentSet);
        }

        collectionIndex = new CollectionIndex(null, indexWords, documentSet, COLLECTION_ID);

        persistence.addDocumentIndex(index2);

        CommonDatabaseOperations.updateCollectionIndex(persistence, collectionIndex);

        assertEquals("use count must not be modified by collection persisting operation"
            , 0, CommonDatabaseOperations.getDocumentUseCount(persistence, sourceId));
        assertEquals("use count must not be modified by collection persisting operation"
            , 0, CommonDatabaseOperations.getDocumentUseCount(persistence, sourceId2));

        assertCollectionIndicesEqual(collectionIndex, CommonDatabaseOperations.getCollectionIndex(persistence,
            COLLECTION_ID));
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#updateCollectionIndex(
     * AbstractDBIndexPersistence, com.topcoder.document.index.CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given collection argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testUpdateCollectionIndexFailureNullCollection() throws Exception {
        try {
            CommonDatabaseOperations.updateCollectionIndex(persistence, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#updateCollectionIndex(
     * AbstractDBIndexPersistence, com.topcoder.document.index.CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given persistence argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testUpdateCollectionIndexFailureNullPersitence() throws Exception {
        try {
            CommonDatabaseOperations.updateCollectionIndex(null, collectionIndex);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#updateCollectionIndex(
     * AbstractDBIndexPersistence, com.topcoder.document.index.CollectionIndex)}.
     * <p/>
     * <b>Failure reason</b>: the given collection argument refers to an unpersisted collection index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testUpdateCollectionIndexFailureUnpersisted() throws Exception {
        try {
            CommonDatabaseOperations.updateCollectionIndex(persistence, collectionIndex);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link CommonDatabaseOperations#increaseDocumentUseCount(AbstractDBIndexPersistence,
     * com.topcoder.document.index.wordsource.WordSourceId, int)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testIncreaseDocumentUseCount() throws Exception {
        persistence.addDocumentIndex(index);
        CommonDatabaseOperations.increaseDocumentUseCount(persistence, sourceId, 1);
        assertEquals("use count increase failed", 1, CommonDatabaseOperations.getDocumentUseCount(persistence,
            sourceId));
        CommonDatabaseOperations.increaseDocumentUseCount(persistence, sourceId, 1);
        assertEquals("use count increase failed", 2, CommonDatabaseOperations.getDocumentUseCount(persistence,
            sourceId));
        CommonDatabaseOperations.increaseDocumentUseCount(persistence, sourceId, -1);
        assertEquals("use count increase failed", 1, CommonDatabaseOperations.getDocumentUseCount(persistence,
            sourceId));
        CommonDatabaseOperations.increaseDocumentUseCount(persistence, sourceId, -1);
        assertEquals("use count increase failed", 0, CommonDatabaseOperations.getDocumentUseCount(persistence,
            sourceId));
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#increaseDocumentUseCount(
     * AbstractDBIndexPersistence, com.topcoder.document.index.wordsource.WordSourceId, int)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testIncreaseDocumentUseCountFailureNullWordSourceId() throws Exception {
        try {
            CommonDatabaseOperations.increaseDocumentUseCount(persistence, null, 1);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#increaseDocumentUseCount(
     * AbstractDBIndexPersistence, com.topcoder.document.index.wordsource.WordSourceId, int)}.
     * <p/>
     * <b>Failure reason</b>: the given persistence argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testIncreaseDocumentUseCountFailureNullPersistence() throws Exception {
        try {
            CommonDatabaseOperations.increaseDocumentUseCount(null, sourceId, 1);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#increaseDocumentUseCount(
     * AbstractDBIndexPersistence, com.topcoder.document.index.wordsource.WordSourceId, int)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument refers to an unpersisted document index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testIncreaseDocumentUseCountFailureUnpersisted() throws Exception {
        try {
            CommonDatabaseOperations.increaseDocumentUseCount(persistence, sourceId, 1);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link CommonDatabaseOperations#getDocumentUseCount(AbstractDBIndexPersistence,
     * com.topcoder.document.index.wordsource.WordSourceId)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetDocumentUseCount() throws Exception {
        persistence.addDocumentIndex(index);
        CommonDatabaseOperations.increaseDocumentUseCount(persistence, sourceId, 1);
        assertEquals("use count get failed", 1, CommonDatabaseOperations.getDocumentUseCount(persistence, sourceId));
        CommonDatabaseOperations.increaseDocumentUseCount(persistence, sourceId, 1);
        assertEquals("use count get failed", 2, CommonDatabaseOperations.getDocumentUseCount(persistence, sourceId));
        CommonDatabaseOperations.increaseDocumentUseCount(persistence, sourceId, -1);
        assertEquals("use count get failed", 1, CommonDatabaseOperations.getDocumentUseCount(persistence, sourceId));
        CommonDatabaseOperations.increaseDocumentUseCount(persistence, sourceId, -1);
        assertEquals("use count get failed", 0, CommonDatabaseOperations.getDocumentUseCount(persistence, sourceId));
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#getDocumentUseCount(
     * AbstractDBIndexPersistence, com.topcoder.document.index.wordsource.WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetDocumentUseCountFailureNullWordSourceId() throws Exception {
        try {
            CommonDatabaseOperations.getDocumentUseCount(persistence, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#getDocumentUseCount(
     * AbstractDBIndexPersistence, com.topcoder.document.index.wordsource.WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given persistence argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetDocumentUseCountFailureNullPersistence() throws Exception {
        try {
            CommonDatabaseOperations.getDocumentUseCount(null, sourceId);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#getDocumentUseCount(
     * AbstractDBIndexPersistence, com.topcoder.document.index.wordsource.WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given wordSourceId argument refers to an unpersisted document index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetDocumentUseCountFailureUnpersisted() throws Exception {
        try {
            CommonDatabaseOperations.getDocumentUseCount(persistence, sourceId);
            fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link CommonDatabaseOperations#getIndexedDocuments(AbstractDBIndexPersistence)} for
     * correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetIndexedDocuments() throws Exception {
        Set set = CommonDatabaseOperations.getIndexedDocuments(persistence);
        assertTrue("indexed documents should be empty", set.isEmpty());

        persistence.addDocumentIndex(index);
        set = CommonDatabaseOperations.getIndexedDocuments(persistence);
        assertEquals("indexed documents size does not match", 1, set.size());
        assertTrue("a document is missing", set.contains(sourceId));

        final WordSourceId sourceId2 = new WordSourceId(SOURCE_IDENTITY2, CLASS_NAME, DELIMITERS, locale);
        final DocumentIndex index2 = new DocumentIndex(sourceId2, words);
        persistence.addDocumentIndex(index2);
        set = CommonDatabaseOperations.getIndexedDocuments(persistence);
        assertEquals("indexed documents size does not match", 2, set.size());
        assertTrue("a document is missing", set.contains(sourceId));
        assertTrue("a document is missing", set.contains(sourceId2));
    }

    /**
     * This method tests the correct failure behavior of {@link CommonDatabaseOperations#getIndexedDocuments(
     * AbstractDBIndexPersistence)}.
     * <p/>
     * <b>Failure reason</b>: the given persistence is <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetIndexedDocumentsFailureNullPersistence() throws Exception {
        try {
            CommonDatabaseOperations.getIndexedDocuments(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        persistence = getPersistenceInstance();
        locale = Locale.getDefault();
        sourceId = new WordSourceId(SOURCE_IDENTITY, CLASS_NAME, DELIMITERS, locale);
        words = new HashMap();
        final Set sourceIds = new HashSet();
        sourceIds.add(sourceId);
        final HashMap collectionWords = new HashMap();
        final Collator collator = Collator.getInstance(locale);
        CollationKey key = collator.getCollationKey(WORD);
        final ArrayList indices = new ArrayList();
        indices.add(new Integer(3));
        indices.add(new Integer(7));
        words.put(key, indices);
        collectionWords.put(key, sourceIds);
        key = collator.getCollationKey(WORD2);
        words.put(key, indices);
        collectionWords.put(key, sourceIds);
        key = collator.getCollationKey(WORD3);
        indices.add(new Integer(9));
        words.put(key, indices);
        collectionWords.put(key, sourceIds);
        index = new DocumentIndex(sourceId, words);
        final HashSet documents = new HashSet();
        documents.add(sourceId);
        collectionIndex = new CollectionIndex(null, collectionWords, documents, COLLECTION_ID);
        connection = persistence.getConnection();
    }

    /**
     * This method creates the instance to be used as connection factory and for persisting documents (the only
     * operation not common between the persistence implementations) in the test cases.
     *
     * @return the persistence instance to be used in test cases.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected abstract AbstractDBIndexPersistence getPersistenceInstance() throws Exception;

    /**
     * This method does the cleanup after test run.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void tearDown() throws Exception {
        connection.commit();
        connection.close();
        super.tearDown();
    }
}
